% heat_statchange.m - Produce a heat map of the % change in a statistic
% between two LHS runs (typically an intervention compared to baseline (no
% intervention)
% WARNING: This code assumes that the two LHS runs are directly comparable
% in terms of numbering of samples, ordering of samples, ranges of
% parameters etc. 
%
% With the updated lhs_framework (and untested identical updates to lhs_sliced)
% random number generator code (written Jan 2013, pushed Apr 2013) this should
% now be the case

function figures = heat_statchange(param1,p1_box,p1_fn,p1axis,param2,p2_box,p2_fn,p2axis,stats,stat_to_vec_fn,c_axis,varargin)
% Test to see if any experiments in varargin    
    if ~(length(varargin) ==2) % If none bye-bye
        error('plot_statparams:experiments', 'Two experiments must be provided');
    end

    % Statistics manipulation: We run these tests on first_exp, and ASSUME
    % the same holds for the second experiment (varargin{2})
    
     first_exp = varargin{1};
     second_exp = varargin{2};
          
     stat_names = first_exp.stat_names;
     % the number of statistics is the size of
     % the final dimension of the stats matrix
     stats_dim = size(first_exp.stats);
     stats_available = stats_dim(end);
     
     if ischar(stats)
         stats = {stats};
     end
     if iscellstr(stats)
         [~, stat_nums] = ismember(stats, stat_names);
         if any(stat_nums < 1)
             error('plot_statistics:stats', 'Invalid statistics name');
         end
     elseif isnumeric(stats)
         stat_nums = stats;
         if any(stat_nums > stats_available)
            error('plot_statistics:stats', 'Invalid statistics index');
         end
     else
         error('plot_statistics:stats', 'Invalid statistics parameter');
     end
    
     % Parametric manipulation
     
     param_names=fieldnames(first_exp.parameter_samples);
     param_dim=size(param_names);
     param_available=param_dim(end);
     
     if ischar(param1)
         param1={param1};
     end
     
     if iscellstr(param1)
         [~,param_nums] = ismember(param1,param_names);
     end
     if any(param_nums < 1)
             error('plot_paramstats3D:param1', 'Invalid parameter index');
     end
 
 if ischar(param2)
         params={param2};
     end
     
     if iscellstr(param2)
         [~,param_nums] = ismember(param2,param_names);
     end
     if any(param_nums < 1)
             error('plot_paramstats3D:param1', 'Invalid parameter index');
     end
     
     first_exp=varargin{1};
     if ~isfield(first_exp,'stat_layout')
         stat_entries = stat_nums;
         stat_name = first_exp.stat_names{stat_nums};
     else % stats are a structure so stat_nums(i) may refer to multiple entries in stats_dim
         seq_of_entries1 = structfun(@(x) x(1), first_exp.stat_layout);
         seq_of_entries2 = structfun(@(x) x(1), second_exp.stat_layout);
         cum_seq_of_entries1 = [0 ; cumsum(seq_of_entries1(1:end-1))];
         cum_seq_of_entries2 = [0 ; cumsum(seq_of_entries2(1:end-1))];
         stat_entries1 = (1:seq_of_entries1(stat_nums)) + cum_seq_of_entries1(stat_nums);
         stat_entries2 = (1:seq_of_entries2(stat_nums)) + cum_seq_of_entries2(stat_nums);
         stat_name = stats;
     end

	% Create a new figure
    figures = figure();
    set(gca,'fontsize',16)
    set(gca,'XMinorTick','on')
    set(gca,'XMinorGrid','on')
    
    param1_data=p1_fn(getfield(first_exp.parameter_samples, char(param1))); % Get the param data
    param2_data=p2_fn(getfield(first_exp.parameter_samples, char(param2)));
    stat_data = cell(1,length(varargin));
    for i = 1:length(varargin)
        exp=varargin{i};	% Pick out an experiment
        if (i == 1)
            stat_entries = stat_entries1;
            stat_vfn = stat_to_vec_fn{1};
        else
            stat_entries = stat_entries2;
            stat_vfn = stat_to_vec_fn{2};
        end
        stat_data{i} = stat_vfn(exp.stats(:, :, stat_entries)); % Get the stat data
    end 
    
    % Now generate percentage change
    change_data = 100*((stat_data{1}-stat_data{2})./stat_data{2});
    % We want the mean percentage change so pass @(x) mean(x) in next line
    [xd yd zd] = mpdmegamean_bb(param1_data,param2_data,change_data,@(x) mean(x),p1_box,p1axis,p2_box,p2axis);
    % Variance in % change(!)
    %[xd yd zd] = mpdmegamean_bb(param1_data,param2_data,change_data,@(x) var(x),p1_box,p1axis,p2_box,p2axis);
    
    surf(xd,yd,zd); % Plot them up
    if ~(c_axis == -1)
        caxis(c_axis)
    end
    
    shading interp
    axis tight
    if strcmp(param1,'R0')
        xlabel('R0')
    elseif strcmp(param1,'O')
        xlabel('log10(Stockpile)')
    else
        xlabel(param1)
    end
    if strcmp(param2,'alpha_m')
        ylabel('\alpha_m')
    elseif (strcmp(param2,'MaxAVtoContacts_Ph1') || strcmp(param2,'MaxAVtoTreat_Ph1'))
        ylabel('log10(Maximum AV Delivery)')
    else
        ylabel(param2)
    end
    zlabel(strcat(exp.stat_names(stat_nums),' (percentage change)'))
    colorbar('Fontsize',16)
    view(0,90)
    if char(p1axis) == 'log'
        set(gca,'XScale','log')
    end
    if char(p2axis) == 'log'
        set(gca,'YScale','log')
    end

% Mega mean
% MPD Wed 8 Dec 2010
%
function [xd yd zd]= mpdmegamean_bb(p1,p2,st,fn,bs1,faxis1,bs2,faxis2)
dat=sortrows([p1',p2',st']);
%disp(['Data size ',num2str(size(dat))]);
%
dat(any(isnan(dat),2),:)=[];
%disp(['Data size post NaN removal ',num2str(size(dat))]);
%
param1=dat(:,1);
param2=dat(:,2);
stat=dat(:,3);
%
maxp1=max(param1);
minp1=min(param1);
maxp2=max(param2);
minp2=min(param2);
md1=max(param1)-min(param1);
md2=max(param2)-min(param2);
%
% Cut points
%cp1=zeros(1,bs1);
%cp2=zeros(1,bs2);
if char(faxis1) =='lin'
    cp1=linspace(minp1,maxp1,bs1);
elseif char(faxis1) == 'log'
    cp1=logspace(log10(minp1),log10(maxp1),bs1);
else
    error('Function 1 illdefined');
end
if char(faxis2) == 'lin'
    cp2=linspace(minp2,maxp2,bs2);
elseif char(faxis2)=='log'
    cp2=logspace(log10(minp2),log10(maxp2),bs2);
else
    error('Function 2 illdefined');
end
%
% First bin is the min of the data set
%cp1(1)=minp1;
%cp2(1)=minp2;
% Remaining bins linearly distributed
%for ii=[2:bs1]
%    cp1(ii)=(md1/(bs1-1))*(ii-bs1)+maxp1;
%end
%for ii=[2:bs2]
%    cp2(ii)=(md2/(bs2-1))*(ii-bs2)+maxp2;
%end
% Last bin is the max of the data set
%cp1(end)=maxp1;
%cp2(end)=maxp2;
%
xd=zeros(bs1-1,bs2-1);
yd=zeros(bs1-1,bs2-1);
zd=zeros(bs1-1,bs2-1);
for ii=[1:bs1-1]
    for jj=[1:bs2-1]
        xind = (param1 > cp1(ii)) & (param1 < cp1(ii+1));
        yind = (param2 > cp2(jj)) & (param2 < cp2(jj+1));
        %
        xd(ii,jj)=mean( param1( xind ) );
        yd(ii,jj)=mean( param2( yind ) );
        zd(ii,jj)=fn( stat(xind & yind ));
    end
end
