function the_figure = prcc(stat,stat_fn,ign_params,expt)
% PRCC Calculator for dummy-parameter LHS runs
%
% Updated from code by Mathew Dafilis (MPD Nov 2011)
%
% Four arguments
% stat == stat name or number
% stat_fn == collapse stat to scalar, use @(x) x if already a scalar
% ign_params == a cell list of parameters to ignore in the calculation, must
% include all dependent parameters, others?
% expt == the lhs expt

% Outputs
% Produces a tornado plot of the prccs
% coeffs == prcc coeffs
% pvals == p-values associated with prcc coeffs
% pnames == list of parameter names associated with prcc

if nargin ~= 4
    error('prcc:', 'incorrect number of arguments');
end
if ~iscellstr(ign_params)
    error('prcc:', 'dependent parameters must be a cell array of strings\n specifying parameter names');
end

% Get parameter names and how many there are
% This doesn't pick up parameters that are held constant, but they don't
% need to be noted for PRCC anyway.
param_names=fieldnames(expt.parameter_samples);
param_count=length(param_names);

% Check to see if ign_params are in the list of model params
if ~ismember(param_names,ign_params)
    error('prcc:', 'dependent params must be contained in the list of model parameters');
end

if ischar(stat)
    [~,stat_index]=ismember(stat,expt.stat_names);
    if stat_index < 1
        error('prcc:', 'invalid statistic name');
    end
else
    error('Code only works for statistics stored as structures')
end

% Following *not* a general solution
% only applies to dummy 1-par runs
seq_of_entries = structfun(@(x) x(1), expt.stat_layout);
cum_seq_of_entries = [0 ; cumsum(seq_of_entries(1:end-1))];
stat_entries = (1:seq_of_entries(stat_index)) + cum_seq_of_entries(stat_index);

stat_v = stat_fn(expt.stats(1,:,stat_entries));

% Set up scratch array to be filled by parameters
param_vals = zeros(param_count,length(stat_v));

param_inds=[];

for ii= 1:param_count
    % Take a parameter name
    pname=param_names{ii};
    % Take the data for this parameter
    pvals=expt.parameter_samples.(pname);
    % We need to collapse the paramters to scalar
    % Hard code this as mean for now
    if (size(pvals,3) > 1)
        pvals=mean(pvals,3);
    end
    % Determine the parameter's range
    range=max(pvals)-min(pvals);
    % If the parameter varies ie range > 0 and the parameter is not a
    % dependent parameter then save the parameter's index and its set of
    % values
    if range > 0 && ~ismember(pname, ign_params);
        param_inds=[param_inds, ii];
        param_vals(ii,:)=pvals;
    end
end

% Remove the non-varying and exluded parameters
param_valse = param_vals(param_inds,:);
param_namese = param_names(param_inds);
param_counte = length(param_inds);

rv=[];
pv=[];
pn=[];

% Loop over the non-constant non-dep params we have
for ii = 1:param_counte
    % Save the original data matrix
    param_valse_tmp=param_valse;
    % Save the iith parameter
    prm_vals=param_valse_tmp(ii,:);
    
    % Clear the iith parameter from the param_valse_tmp matrix
    param_valse_tmp(ii,:)=[];
    % Join the parameter of interest and the statistic of interest 
    difmat =[prm_vals ; stat_v];
    
    % Form the partial-rank-correlation-coefficient between the parameter of
    % interest and the statistic of interest (difmat) controlling for all of
    % the other parameters (param_valse_tmp)
    [rho,pval]=partialcorr(difmat', param_valse_tmp','type','Spearman');
    
    % rho and pval are in a 2x2 symmetric matrix. We want one of the
    % off-diagonal elements.
    trv=rho(1,2);
    tpv=pval(1,2);
    % Check for pesky NaNs and also check to make sure prcc is stat
    % significant at 0.05 or less
    if ~isnan(trv) && ~isnan(tpv) && tpv<=0.05
        % Save pvals and rhos, and parameter names that pass the
        % above tests
        rv=[rv; trv];
        pv=[pv ; tpv];
        pn=[pn; param_namese(ii)];
    end
end

if ~isempty(rv)
    % Sort the prcc values based on magnitude keeping the reordering details in
    % ixx.
    [sorted_rv,ixx]=sort(rv);
    
    %% Prepare tornado plot
    the_figure = figure();
    % Plot sorted rho values from most positive to least negative
    hh = barh(sorted_rv);
    set(gca,'YTickLabel',pn(ixx));
    set(hh,'FaceColor',[0.5 0.5 0.5]); % Grey bars
    xlabel('PRCC');
    ylabel('Parameter');
    xlim([-1 1]);
    tt= ['Tornado plot of PRCC with respect to ',expt.stat_names(stat_index)];
    title(tt);
else
    fprintf('No stat. sig. parameters\n')
end