%% Model Equations
% Calculates the change in the state of the model.

%%
% The change in state of the model is derived from the current time, the current
% model state, and the model parameters.
%
function [state_delta] = discrete_rhs(time, state, params)

%%
% Events that occur during the epidemic (ie, the beginning and the end of
% the Contain phase, and the complete depletion of the stockpile) are
% recorded in the global record |events|.
%
global events;

%%
% Scale time-dependent parameters to be per time-step
%
%
params.omega = params.omega / events.timescale;
params.gamma = params.gamma / events.timescale;
params.delta = params.delta / events.timescale;
params.SchoolStart = params.SchoolStart * events.timescale;
params.SchoolDuration = params.SchoolDuration * events.timescale;
params.VaccineStart = params.VaccineStart * events.timescale;
params.VaccineDeliveryRate = params.VaccineDeliveryRate / events.timescale;
params.HCWStop = params.HCWStop * events.timescale;
params.HCWRate_Ph1 = params.HCWRate_Ph1 / events.timescale;
params.HCWRate_Ph2 = params.HCWRate_Ph2 / events.timescale;
params.ContainDuration = params.ContainDuration * events.timescale;
params.ContainCaseLoad = params.ContainCaseLoad * 365 / events.timescale;
params.MaxAVtoTreat_Ph1 = params.MaxAVtoTreat_Ph1 * 365 / events.timescale;
params.MaxAVtoTreat_Ph2 = params.MaxAVtoTreat_Ph2 * 365 / events.timescale;
params.MaxAVtoContacts_Ph1 = params.MaxAVtoContacts_Ph1 * 365 / events.timescale;
params.MaxAVtoContacts_Ph2 = params.MaxAVtoContacts_Ph2 * 365 / events.timescale;
params.Dc = params.Dc * 365 / events.timescale;

% Round up if we've crossed into negative values
state(state < 0) = 0;

% Fix shape of params.alpha_m (dep distribution limitation)
params.alpha_m = params.alpha_m';

%%
% The model state is stored as a vector; each state variable is extracted
% from the vector and associated with a meaningful name. Some of these
% variables (eg, |RI_*| and |RA_*|) are not actually used in the model
% equations, but are nevertheless defined in order to demonstrate the
% organisation of the model state vector.
%
n = length(params.pi);
S = state(1:n);
S_v = state(n+1:2*n);
C_p_w = state(2*n+1:3*n);
C_np_w = state(3*n+1:4*n);
C_p_r = state(4*n+1:5*n);
C_np_r = state(5*n+1:6*n);
E_p_w = state(6*n+1:7*n);
E_np_w = state(7*n+1:8*n);
E_p_r = state(8*n+1:9*n);
E_np_r = state(9*n+1:10*n);
I_p_w = state(10*n+1:11*n);
I_npt_w = state(11*n+1:12*n);
I_npnt_w = state(12*n+1:13*n);
I_p_r = state(13*n+1:14*n);
I_np_r = state(14*n+1:15*n);
A_p_w = state(15*n+1:16*n);
A_np_w = state(16*n+1:17*n);
A_p_r = state(17*n+1:18*n);
A_np_r = state(18*n+1:19*n);
RI_p_w = state(19*n+1:20*n);
RI_np_w = state(20*n+1:21*n);
RI_p_r = state(21*n+1:22*n);
RI_np_r = state(22*n+1:23*n);
RA_p_w = state(23*n+1:24*n);
RA_np_w = state(24*n+1:25*n);
RA_p_r = state(25*n+1:26*n);
RA_np_r = state(26*n+1:27*n);
%CumInc = state(27*n+1:28*n);
%CumIncP = state(28*n+1:29*n);
O = state(29*n+1);
%O_pPh1 = state(29*n+2);
%O_tPh1 = state(29*n+3);
%O_pPh2 = state(29*n+4);
%O_tPh2 = state(29*n+5);
%O_tSev = state(29*n+6);
%O_tMld = state(29*n+7);
%O_tWst = state(29*n+8);
%VaccineesFromS = state(29*n+9:30*n+8);
TotalVaccinees = state(30*n+9:31*n+8);

% Population sizes for convenience
Ns = params.pi * params.N;

% Presentation rates derived from the model parameters $\eta$ and $\alpha_M$.
if isfield(params, 'eta_relscale')
    %% Important: non-uniform values for 'eta_relscale' will change the
    %% overall proportion of infections in each strata that present, since
    %% 'alpha_m' is the proportion of *remaining* (i.e., non-severe)
    %% infections that present.
    params.eta = params.eta .* params.eta_relscale;
end
alpha_np_w = params.eta + params.alpha_m .* (1 - params.eta);
alpha_p_w = params.eta + params.proph_relpathogenicity * params.alpha_m .* (1 - params.eta);
alpha_r = alpha_np_w;

% Some important parameter matricies: (n,n) repeats of (n,1) vectors
Ei = repmat(params.e_i,1,n);
Et = repmat(params.e_t,1,n);

%%
% Presentation rates
%
presn_rate_p = alpha_p_w .* params.omega .* E_p_w;
presn_rate_np = alpha_np_w .* params.omega .* E_np_w;
presn_rate_p_r = alpha_r .* params.omega .* E_p_r;
presn_rate_np_r = alpha_r .* params.omega .* E_np_r;
presn_rate = presn_rate_p + presn_rate_np + presn_rate_p_r + presn_rate_np_r;

%%
% Determine whether the model is currently in the pre-Contain phase, the
% Contain phase, or the post-Contain phase.
%
phase = which_phase(params, time, sum(presn_rate));

%%
% Set phase-specific model parameters to the correct values for the
% current phase.
%
if (phase == 0)
    % parameter values for the initial (pre detection) phase
    HCWRate = 0;
    MaxAVtoContacts = 0;
    MaxAVtoTreat = 0;
elseif (phase == 1)
    % parameter values for the 1st response phase
    MaxAVtoContacts = params.MaxAVtoContacts_Ph1;
    MaxAVtoTreat = params.MaxAVtoTreat_Ph1;
    HCWRate = params.HCWRate_Ph1;
elseif (phase == 2)
    % parameter values for the 2nd response phase
    MaxAVtoContacts = params.MaxAVtoContacts_Ph2;
    MaxAVtoTreat = params.MaxAVtoTreat_Ph2;
    HCWRate = params.HCWRate_Ph2;
end

%%
% Determine the degree to which treatment and prophylaxis are reaching the
% relevant fraction of the population, and the depletion of the antiviral
% stockpile.
%
if (phase == 0)
    % no treatment or prophylaxis in the initial phase
    epsilon = zeros(n,1); psi = zeros(n,1); use_treat = 0; use_proph = 0;
    use_treat_severe = 0;
    use_treat_mild = 0;
    use_treat_waste = 0;
else
    [psi, epsilon, use_treat, use_proph, use_treat_severe, use_treat_mild, ...
     use_treat_waste] = ...
        pt_values(params, MaxAVtoContacts, MaxAVtoTreat, ...
        presn_rate_p, presn_rate_np, time, phase);
end

%%
% Stop treating health-care workers once the deadline |HCWStop| has been
% reached.
%
if (time > params.HCWStop)
    HCWRate = 0;
end

%%
% Determine the vaccination rate based on delivery and success rates,
% taking into account vaccine availability (|VaccineStart|) and the size
% of the unvaccinated population (|N - TotalVaccinees|).
if (time <= params.VaccineStart)
    VaccineSuccessRate = zeros(n,1);
    VaccineDeliveryRate = zeros(n,1);
else
    mask = (TotalVaccinees < (params.N * params.pi));
    % mask (binary) indicates which age groups still have unvaccinated
    % individuals.
    % VSuccessRate and VDeliveryRate are set to zero for "completed" age
    % groups
    VaccineSuccessRate = params.VaccineDeliveryRate ...
                         .* (1 - params.VaccineFailureRate) .* mask;
    VaccineDeliveryRate = params.VaccineDeliveryRate .* mask;
end

% Short name for vaccine effect...
e_v = params.VaccineSusceptibility;

%%
% Check whether the stockpile has been depleted at this point in time (ie,
% it has not already been depleted, but the size of the remaining
% stockpile is 0).
%
if (events.stockpileExpired == 0 && O <= 0)
    events.stockpileExpired = 1;
    events.time_stockpileExpired = time;

    % record the total # and proportion of all cases that are drug resistant
    resCases = E_p_r + E_np_r + I_p_r + I_np_r + A_p_r + A_np_r ...
        + RI_p_r + RI_np_r + RA_p_r + RA_np_r;
    nrmCases = E_p_w + E_np_w + I_p_w + I_npt_w + I_npnt_w + A_p_w + A_np_w ...
        + RI_p_w + RI_np_w + RA_p_w + RA_np_w;

    events.resCasesAtExpiry = resCases;
    events.resPropnAtExpiry = resCases ./ (nrmCases + resCases);
end

%%
% If the stockpile has expired, halt the distribution of antivirals for
% treatment, prophylaxis of infected cases, and for prophylaxis of
% healthcare workers.
if (events.stockpileExpired > 0)
    epsilon = zeros(n,1);
    psi = zeros(n,1);
    use_treat = 0;
    use_proph = 0;
    use_treat_severe = 0;
    use_treat_mild = 0;
    use_treat_waste = 0;
    HCWRate = 0;
end

%%
% Convenience factors for calculating the forces of infection.
%
% Mixing between Children and Adults required here...
% See McCaw JM & McVernon J, Multi-class antiviral stockpile extinction
% deterministic model working paper (March 2006).
% Available at ~jamesm/population_health/influenza_research/ ...
% stockpile_extinction/multiclass_AVs (including sample MATLAB code)

% School based measures modify M_{11} relative to M_{nn}
%if ((time>params.SchoolStart) && (time<(params.SchoolStart+params.SchoolDuration)))
%    params.M(1,1) = params.M(1,1)*(1-params.SchoolEfficacy);
%end

lambda_p_w = params.R0scaling * (Ei .* (params.kappa/events.timescale) .* params.tpm) * ((I_p_w + params.chi .* A_p_w) ./ Ns);
lambda_npt_w = params.R0scaling * (Et .* (params.kappa/events.timescale) .* params.tpm) * (I_npt_w ./ Ns);
lambda_npnt_w = params.R0scaling * ((params.kappa/events.timescale) .* params.tpm) * ((I_npnt_w + params.chi .* A_np_w) ./ Ns);

lambda_w = lambda_p_w + lambda_npt_w + lambda_npnt_w;

lambda_r = (params.R0scaling * params.phi) * ((params.kappa/events.timescale) .* params.tpm) * ...
           ((I_p_r + params.chi .* A_p_r + I_np_r + params.chi .* A_np_r) ./ Ns);

%%
% More convenience factors for calculating the forces of infection.
%
Sum_of_C_w = C_p_w + C_np_w;
Sum_of_C_r = C_p_r + C_np_r;

mask = (Sum_of_C_w == 0);

% What should these really equal - it doesn't seem to matter...
Theta_p_w = S;
Theta_np_w= S;
Theta_v_p_w = S_v;
Theta_v_np_w= S_v;
Theta_p_w(~mask) = params.e_s(~mask) .* C_p_w(~mask) .* S(~mask) ./ Sum_of_C_w(~mask);
Theta_np_w(~mask) = C_np_w(~mask) .* S(~mask) ./ Sum_of_C_w(~mask);
Theta_v_p_w(~mask) = params.e_s(~mask) .* C_p_w(~mask) .* e_v(~mask) .* S_v(~mask) ./ Sum_of_C_w(~mask);
Theta_v_np_w(~mask) = C_np_w(~mask) .* e_v(~mask) .* S_v(~mask) ./ Sum_of_C_w(~mask);

mask = (Sum_of_C_r == 0);
% What should these really equal - it doesn't seem to matter...
Theta_p_r = S;
Theta_np_r = S;
Theta_v_p_r = S_v;
Theta_v_np_r = S_v;
Theta_p_r(~mask) = C_p_r(~mask) .* S(~mask) ./ Sum_of_C_r(~mask);
Theta_np_r(~mask) = C_np_r(~mask) .* S(~mask) ./ Sum_of_C_r(~mask);
Theta_v_p_r(~mask) = C_p_r(~mask) .* e_v(~mask) .* S_v(~mask) ./ Sum_of_C_r(~mask);
Theta_v_np_r(~mask) = C_np_r(~mask) .* e_v(~mask) .* S_v(~mask) ./ Sum_of_C_r(~mask);

resCases = E_p_r + E_np_r + I_p_r + I_np_r + A_p_r + A_np_r ...
           + RI_p_r + RI_np_r + RA_p_r + RA_np_r;


if (params.phi ~= 0) | (Sum_of_C_r ~= 0) | (resCases ~= 0)
  error('Drug resistant strains should be disabled');
end

if (params.HCWRate_Ph1 ~= params.HCWRate_Ph2)
  error('HCWRate varies across phases');
end
if (params.MaxAVtoTreat_Ph1 ~= params.MaxAVtoTreat_Ph2)
  error('MaxAVtoTreat varies across phases');
end
if (params.MaxAVtoContacts_Ph1 ~= params.MaxAVtoContacts_Ph2)
  error('MaxAVtoContacts varies across phases');
end


%%
% Calculate flow rates from E to I, taking into account deterministic and/or
% stochastic emergence and spread of a drug-resistant strain
%

    function rate = stoch_flow(People, Pr)

            % Initialise output to ensure shape (size(People)) is maintained.
            rate = zeros(size(People));

            People = floor(People);

            % When the expected (yearly) rate is greater than the number of
            % people, stochastically choose the number of people to move in a
            % single timestep and then scale this to get the yearly rate.

            % Set rates to zero when there are no people or zero probability
            zero_inds = or( People < 1, Pr == 0 );
            rate(zero_inds) = 0;

            if any(Pr > 1)
                fprintf(1, 'ERROR: Pr > 1\n');
            end

            % Use the Poisson distribution (the law of rare events)
            % when People > 1e3 and Pr < 1e-4 (ie, B(n,p) ~ P(np))
            % NOTE: delT must be smaller than all rate constants
            poiss_inds = and( People >= 1e3, ~zero_inds);
            if nnz(poiss_inds) > 0
                rate(poiss_inds) = poissrnd(People(poiss_inds) .* Pr(poiss_inds));
            end

            % Use binomial selection in all other cases
            bino_inds = and(~zero_inds, ~poiss_inds);
            if nnz(bino_inds) > 0
                rate(bino_inds) = binornd(People(bino_inds), Pr(bino_inds));
            end
    end

%
% Calculate deterministic flows required to calculate all the stochastic flows
%
out_Enpw = params.omega .* E_np_w;
out_Enpw_I = out_Enpw .* alpha_np_w;
out_Enpw_Tr = out_Enpw_I .* psi;
out_Enpw_NoTr = out_Enpw_I .* (1 - psi);
out_Enpw_A = out_Enpw .* (1 - alpha_np_w);
out_Epw = params.omega .* E_p_w;

%
% Calculate all of the stochastic flows in one call, to reduce overhead
%
People = [E_np_r E_p_r out_Enpw_Tr out_Enpw_NoTr out_Enpw_A out_Epw ...
    S S S_v S_v C_p_r C_np_r I_p_r I_np_r A_p_r A_np_r];

mask = (S==0);
Xi_pr = zeros(n,1);
Xi_npr = zeros(n,1);
Xi_pr(~mask) = lambda_r(~mask) .* Theta_p_r(~mask) ./ S(~mask);
Xi_npr(~mask) = lambda_r(~mask) .* Theta_np_r(~mask) ./ S(~mask);

mask = (S_v == 0);
Xiv_pr = zeros(n,1);
Xiv_npr = zeros(n,1);
Xiv_pr(~mask) = lambda_r(~mask) .* Theta_v_p_r(~mask) ./ S_v(~mask);
Xiv_npr(~mask) = lambda_r(~mask) .* Theta_v_np_r(~mask) ./ S_v(~mask);

Pr = [params.omega params.omega repmat(params.rho_t,n,1) ...
    repmat(params.rho_n,n,1) repmat(params.rho_n,n,1) repmat(params.rho_p,n,1) ...
    Xi_pr ...
    Xi_npr ...
    Xiv_pr ...
    Xiv_npr ...
    params.delta params.delta ...
    params.gamma params.gamma params.gamma params.gamma];
stoch_rates = stoch_flow(People, Pr);
out_Enpr = stoch_rates(:,1);
out_Epr = stoch_rates(:,2);
out_Enpw_Tr_Inpr = stoch_rates(:,3);
out_Enpw_NoTr_Inpr = stoch_rates(:,4);
out_Enpw_Anpr = stoch_rates(:,5);
out_Epw_r = stoch_rates(:,6);
out_S_Epr = stoch_rates(:,7);
out_S_Enpr = stoch_rates(:,8);
out_Sv_Epr = stoch_rates(:,9);
out_Sv_Enpr = stoch_rates(:,10);
out_Cpr_delta = stoch_rates(:,11);
out_Cnpr_delta = stoch_rates(:,12);
out_Ipr = stoch_rates(:,13);
out_Inpr = stoch_rates(:,14);
out_Apr = stoch_rates(:,15);
out_Anpr = stoch_rates(:,16);

% The flow rates out of E_p_r states
out_Epr_Ipr = floor(out_Epr .* alpha_r);
out_Epr_Apr = out_Epr - out_Epr_Ipr;
out_Enpr_Inpr = floor(out_Enpr .* alpha_r);
out_Enpr_Anpr = out_Enpr - out_Enpr_Inpr;

% The flow rates from E_np_w to I states (ie, presentations)
out_Enpw_Inptw = out_Enpw_Tr - out_Enpw_Tr_Inpr;
out_Enpw_Inpntw = out_Enpw_NoTr - out_Enpw_NoTr_Inpr;
out_Enpw_Inpr = out_Enpw_Tr_Inpr + out_Enpw_NoTr_Inpr;

% The flow rates from E_np_w to A states (ie, non-presentations)
out_Enpw_Anpw = out_Enpw_A - out_Enpw_Anpr;

% The flow rates from E_p_w to wild-strain and resistant-strain states
out_Epw_Ipr = floor(out_Epw_r .* alpha_r);
out_Epw_Apr = out_Epw_r - out_Epw_Ipr;
out_Epw_w = out_Epw - out_Epw_r;
out_Epw_Ipw = out_Epw_w .* alpha_p_w;
out_Epw_Apw = out_Epw_w .* (1 - alpha_p_w);

% The flow rates out of C_p_r and C_np_r
out_Cpr = out_Cpr_delta + out_S_Epr + out_Sv_Epr;
out_Cnpr = out_Cnpr_delta + out_S_Enpr + out_Sv_Enpr;

% NOTE: transpose essential for creation of contacts
% The flow rates in to C_p_r and C_np_r
new_Cr = (params.kappa'/365) * (out_Epr + out_Enpr + ...
    out_Enpw_Inpr + out_Enpw_Anpr + out_Epw_r);
new_Cpr = floor ( epsilon .* ((params.kappa'/365) * (out_Epr_Ipr + out_Enpr_Inpr + ...
    out_Epw_Ipr + out_Enpw_Inpr)) );
new_Cnpr = new_Cr - new_Cpr;

% The flow rates out of C_p_w and C_np_w
out_Cpw = params.delta .* C_p_w + ...
    lambda_w .* (Theta_p_w + Theta_v_p_w);
out_Cnpw = params.delta .* C_np_w + ...
    lambda_w .* (Theta_np_w + Theta_v_np_w);

% The flow rates in to C_p_w and C_np_w
new_Cw = (params.kappa'/365) * (out_Enpw_Inptw + out_Enpw_Inpntw + ...
    out_Enpw_Anpw + out_Epw_Ipw + out_Epw_Apw);
new_Cpw = floor( epsilon .* ((params.kappa'/365) * (out_Epw_Ipw + ...
    out_Enpw_Inptw + out_Enpw_Inpntw)) );
new_Cnpw = new_Cw - new_Cpw;

%%
% The depletion of the susceptible population is driven by the forces of
% infection. Contacts of people infected with the wild or drug-resistant
% strain become exposed; vaccination reduces this susceptibility.
%

S_dot = - (lambda_w .* (Theta_p_w + Theta_np_w) ... % wild strain
    + out_S_Epr + out_S_Enpr) ... % resistant strain
    - VaccineSuccessRate .* S / params.N; % vaccination

S_v_dot = - (lambda_w .* (Theta_v_p_w + Theta_v_np_w) ... % wild strain
    + out_Sv_Epr + out_Sv_Enpr) ... % resistant strain
    + VaccineSuccessRate .* S / params.N; % new vaccinations
%%
% The contact status of persons that have come into contact with infected
% individuals.
%
C_p_w_dot = new_Cpw - out_Cpw;
C_np_w_dot = new_Cnpw - out_Cnpw;
C_p_r_dot = new_Cpr - out_Cpr;
C_np_r_dot = new_Cnpr - out_Cnpr;

%%
% Exposed individuals flow in from the susceptible states, and after the
% duration of their latent phase they become infectious.
%
E_p_w_dot = lambda_w .* ... % new exposures from S and S_v
    (Theta_p_w + Theta_v_p_w) ...
    - out_Epw;  % end of latent phases

E_np_w_dot = lambda_w .* ... % new exposures from S and S_v
    (Theta_np_w + Theta_v_np_w) ...
    - out_Enpw;  % end of latent phases

E_p_r_dot = out_S_Epr + out_Sv_Epr ... % new exposures from S and S_v
    - out_Epr;  % end of latent phases

E_np_r_dot = out_S_Enpr + out_Sv_Enpr ... % new exposures from S and S_v
    - out_Enpr;  % end of latent phases

%%
% Infectious individuals flow in from the exposed states, and after the
% duration of their infectious phase they become recovered. Those that
% presented (|I_*|) may have received prophylaxis or treatment, while
% those that did not present (|A_*|) can only have received prophylaxis.
%
I_p_w_dot = out_Epw_Ipw - params.gamma .* I_p_w;
I_npt_w_dot = out_Enpw_Inptw - params.gamma .* I_npt_w;
I_npnt_w_dot = out_Enpw_Inpntw - params.gamma .* I_npnt_w;

I_p_r_dot = out_Epr_Ipr + out_Epw_Ipr - out_Ipr;
I_np_r_dot = out_Enpr_Inpr + out_Enpw_Inpr - out_Inpr;

CumInc_dot = out_Epw_Ipw + out_Enpw_Inptw + out_Enpw_Inpntw + ...
    out_Epr_Ipr + out_Enpr_Inpr;

CumIncP_dot = out_Epw_Ipw;

A_p_w_dot = out_Epw_Apw - params.gamma .* A_p_w;
A_np_w_dot = out_Enpw_Anpw - params.gamma .* A_np_w;

A_p_r_dot = out_Epr_Apr + out_Epw_Apr - out_Apr;
A_np_r_dot = out_Enpr_Anpr + out_Enpw_Anpr - out_Anpr;

%%
% Recovered individuals flow in from the infectious states.
%
RI_p_w_dot = params.gamma .* I_p_w;
RI_np_w_dot = params.gamma .* (I_npnt_w + I_npt_w);
RI_p_r_dot = out_Ipr;
RI_np_r_dot = out_Inpr;

RA_p_w_dot = params.gamma .* A_p_w;
RA_np_w_dot = params.gamma .* A_np_w;
RA_p_r_dot = out_Apr;
RA_np_r_dot = out_Anpr;

%%
% The depletion of the antiviral stockpile is recorded on a per-phase
% basis.
%
if (events.endPh1 == 0)
    O_pPh1_dot = sum(use_proph);
    O_tPh1_dot = sum(use_treat);
    O_pPh2_dot = 0;
    O_tPh2_dot = 0;
else
    O_pPh1_dot = 0;
    O_tPh1_dot = 0;
    O_pPh2_dot = sum(use_proph);
    O_tPh2_dot = sum(use_treat);
end

O_tSev_dot = sum(use_treat_severe);
O_tMld_dot = sum(use_treat_mild);
O_tWst_dot = sum(use_treat_waste);

%%
% Keep track of the remaining antiviral stockpile and the progress of the
% vaccination effort.
%
O_dot = -(O_pPh1_dot + O_tPh1_dot + O_pPh2_dot + O_tPh2_dot + HCWRate);

VfromS_dot = VaccineSuccessRate .* S / params.N;

VTotal_dot = VaccineDeliveryRate;

%%
% Return the change to model state.
%
state_delta = [ ...
    S_dot; S_v_dot; ...
    C_p_w_dot; C_np_w_dot; C_p_r_dot; C_np_r_dot; ...
    E_p_w_dot; E_np_w_dot; E_p_r_dot; E_np_r_dot; ...
    I_p_w_dot; I_npt_w_dot; I_npnt_w_dot; I_p_r_dot; I_np_r_dot; ...
    A_p_w_dot; A_np_w_dot; A_p_r_dot; A_np_r_dot; ...
    RI_p_w_dot; RI_np_w_dot; RI_p_r_dot; RI_np_r_dot; ...
    RA_p_w_dot; RA_np_w_dot; RA_p_r_dot; RA_np_r_dot; ...
    CumInc_dot; CumIncP_dot; ...
    O_dot; O_pPh1_dot; O_tPh1_dot; O_pPh2_dot; O_tPh2_dot; ...
    O_tSev_dot; O_tMld_dot; O_tWst_dot; ...
    VfromS_dot; VTotal_dot];

% Sanity checks
if (length(state_delta)~=length(state))
    length(state)
    length(state_delta)
    fprintf(1,' ERROR: State vector length not conserved\n');
end
if any(or(isnan(state_delta), isinf(state_delta)))
    fprintf(1, 'ERROR: NaN or Inf at t = %d\n', time);
end
if any(state < 0)
    i = find(state < 0, 1);
    fprintf(1, 'ERROR: Negative state (%d = %f) at t = %d\n', i, state(i), time);
end
end

%%
% Determines the phase that the model is currently in. The possible values
% are: 0 (pre-Contain); 1 (Contain); 2 (post-Contain).
%
function [phase] = which_phase(params, time, presn_rate)
    global events;

    daily_presn_rate = presn_rate * events.timescale / 365;
    if (events.startPh1 == 0)
        % check whether the contain (phase 1) phase should start
        if daily_presn_rate < params.start_intervention_rate;
            phase = 0;
        else
            phase = 1;
            events.startPh1 = 1;
            events.time_startPh1 = time;
        end
    elseif (events.endPh1 == 0)
        % check whether the contain phase should end
        duration_end = events.time_startPh1 + params.ContainDuration;
        time_to_end = time >= duration_end;
        load_to_end = daily_presn_rate >= params.ContainCaseLoad;
        if (params.TimeNotCaseLoad == 1 && time_to_end)
            phase = 2;
            events.endPh1 = 1;
            events.time_endPh1 = time;
            %% fprintf('Starting Phase 2 ...\n');
            %% fprintf('    Treat strata  ');
            %% fprintf('%1d ', params.treat_strata');
            %% fprintf('\n');
            %% fprintf('    Proph strata  ');
            %% fprintf('%1d ', params.proph_strata');
            %% fprintf('\n');
        elseif (params.TimeNotCaseLoad == 0 && load_to_end)
            phase = 2;
            events.endPh1 = 1;
            events.time_endPh1 = time;
        else
            phase = 1;
        end
    else
        % the contain (phase 1) phase has ended
        phase = 2;
    end
end

%%
% Calculates the fraction of presentations that are treated and whose
% contacts are given prophylaxis, taking into account diagnosis and
% prophylaxis constraints. The values |treat| and |proph| are the
% antiviral usage for treatment and prophylaxis, respectively, accounting
% for wastage.
%
function [psi, epsilon, treat, proph, treat_s, treat_m, treat_w] = pt_values(params, ...
    MaxAVtoContacts, MaxAVtoTreat, presn_rate_p, presn_rate_np, time, phase)
    global events;

    % WARNING
    if (params.phi ~= 0)
        error('Capacity constraints do not deal with resistant strains. pt_values() requires extension')
    end

    %% NOTE: presentation rates are 5x1 vectors (i.e., per strata).

    n = length(params.pi);
    %
    % Presentations
    %

    % Overall presentation rate
    presn_rate = presn_rate_p + presn_rate_np;
    % Ensure non-zero presentation rate and delivery rates
    if (MaxAVtoContacts == 0 && MaxAVtoTreat == 0) || sum(presn_rate) * events.timescale < 365
        % Although MaxAVtoTreat does not apply to severe (ie, hospitalised)
        % cases, setting both rates to zero signals no intervention at all.
        % The presentation rate is forced to be >= 365 to ensure that there is
        % at least one presentation per day.
        psi = zeros(n,1);
        epsilon = zeros(n,1);
        treat = 0;
        proph = 0;
        treat_s = 0;
        treat_m = 0;
        treat_w = 0;
        return;
    end
    % Percentage of all ILI presentations that are infected
    % (presn_rate is converted from per year to per week)
    I_pcnt = min(0.65, max(0.03, sum(presn_rate) * 3e-5 * events.timescale * 7/365));
    % Percentage of infected cases that have not received prophylaxis
    NP_pcnt = zeros(n,1);
    mask = (presn_rate==0);
    NP_pcnt(~mask) = presn_rate_np(~mask) ./ presn_rate(~mask);
    % Presentation rate of the mild cases
    mild_presn_rate = (presn_rate_p .* ...
        params.proph_relpathogenicity .* params.alpha_m .* (1 - params.eta) ./ ...
        (params.eta + params.proph_relpathogenicity * params.alpha_m .* (1 - params.eta) )) ...
        + ...
        (presn_rate_np .* ...
        params.alpha_m .* (1 - params.eta) ./ ...
        (params.eta + params.alpha_m .* (1 - params.eta) ));

    % Presentation rate of the severe cases
    sev_presn_rate = presn_rate - mild_presn_rate;

    %
    % Diagnosis
    %

    % Calculate the maximal diagnosis rate.
    % NOTE: params.Dc is the sum of the GP and ED consultation capacities.
    diag_cap_exceeded = (sum(mild_presn_rate) / I_pcnt) > params.Dc;
    if diag_cap_exceeded && (events.time_diagCapExceeded < 0)
        events.time_diagCapExceeded = time;
        fprintf('Diagnostic capacity exceeded at %f during phase %d\n', ...
                time, phase);
        % Check whether to use syndromic diagnosis
        % TODO: generalise to params.switchStrategy and params.newStrategy
        %       eg, params.newStrategy = 'PoC (local)'
        if params.switchToSyndromic
            events.useSyndromic = 1;
            fprintf('Switching to syndromic surveillance.\n')
        end
    end
    if events.useSyndromic
        params = diagnosis_regime(params, 'syndromic');
    end
    max_diag_rate = params.Dc;
    % Rate of diagnosis (mild cases)
    % Single strata population:
    %dDiag = min(max_diag_rate, mild_presn_rate / I_pcnt);
    % Multi strata population (all individuals equally eligible):
    desired = mild_presn_rate / I_pcnt;
    if (sum(desired) <= max_diag_rate)
        dDiag = desired;
    else % And of course, this only triggers if we don't switch to syndromic
        if ~isfield(params,'Dc_prioritygroups')
            dDiag = desired * max_diag_rate/sum(desired);
        else
            dDiag = priority_allocation(desired,params.Dc_prioritygroups,max_diag_rate);
        end
    end

    % Rate of diagnosis of infected mild cases
    dDiag_I = dDiag * I_pcnt;
    % Rate of diagnosis of non-infected cases
    dDiag_NI = dDiag * (1 - I_pcnt);
    % Rate of positive diagnosis results
    dDiag_Pos = dDiag_I * params.Sn + dDiag_NI * (1 - params.Sp);

    %
    % Treatment
    %

    % Rate of treatment given out
    % We assume that all diagnosed cases, whether diagnosed correctly or not,
    % have an equal likelihood of having already received prophylaxis
    % We also introduce mu, the compliance, i.e. probability of choosing
    % to treat (and follow up contacts) given the diagnosis.
    % Single strata population:
    %dTreat_Given = min(MaxAVtoTreat, NP_pcnt * dDiag_Pos);
    % Multi strata population (all individuals equally eligible):
    desired = NP_pcnt .* dDiag_Pos .* params.mu;
    % Only provide treatment to the targeted strata.
    % NOTE: this *only* applies during the targeted intervention phase.
    if phase == 2
        desired = desired .* params.treat_strata;
    end
    if (sum(desired) <= MaxAVtoTreat)
        dTreat_Given = desired;
    else
        if ~isfield(params,'Treat_prioritygroups')
            dTreat_Given = desired * MaxAVtoTreat/sum(desired);
        else
            dTreat_Given = priority_allocation(desired,params.Treat_prioritygroups,MaxAVtoTreat);
        end
    end

    % Percentage of diagnosed cases that are infected mild cases
    DI_pcnt = zeros(n,1);
    mask = (dDiag_Pos==0);
    DI_pcnt(~mask) = dDiag_I(~mask) * params.Sn ./ dDiag_Pos(~mask);
    % Rate of treatment given to infected mild cases
    dTreat_I = dTreat_Given .* DI_pcnt;
    % Rate of effective treatment given to infected mild cases
    % scale efficacy by (eta - alpha) / alpha and add P x eta / alpha
    dTreat_Eff = dTreat_I .* (params.e_dt .* params.frac_gp + (1 - params.frac_gp));
    % All severe cases and some mild cases receive treatment
    psi = zeros(n,1);
    mask = (presn_rate==0);

    psi_wrong(~mask) = (dTreat_Eff(~mask) + params.Rxsev(~mask).* sev_presn_rate(~mask)) ./ presn_rate(~mask);

    psi(~mask) = (mild_presn_rate(~mask) ./ presn_rate(~mask)) .* dTreat_Eff(~mask) ./ ( NP_pcnt .* mild_presn_rate(~mask)) + ...
        (sev_presn_rate(~mask) ./ presn_rate(~mask)) .* NP_pcnt;

    % The number of AV doses dispensed for treatment (including "wastage").
    treat = dTreat_Given + params.Rxsev .* NP_pcnt .* sev_presn_rate;
    % Treatment given to mild cases (not equivalent to distribution, which
    % can include non-influenza ILI cases).
    treat_m = sum(dTreat_I);
    % Treatment given to mild cases of other respiratory illnesses.
    treat_w = sum(dTreat_Given) - treat_m;
    % Treatment given to severe cases.
    treat_s = sum(params.Rxsev .* NP_pcnt .* sev_presn_rate);

    %
    % Prophylaxis
    %

    % The rate of prophylaxis given out to contacts of positive diagnoses
    % Single strata population:
    %dProph_Given = min(MaxAVtoContacts, (dDiag_Pos + sev_presn_rate) ...
    %    * kappa * params.sigma);
    % Multi strata population (all individuals equally eligible):
    % Note, transpose of kappa very important here...
    desired = params.sigma .* ((params.kappa'/365) * (dDiag_Pos .* params.mu + sev_presn_rate));
    % Only provide prophylaxis to the targeted strata.
    % NOTE: this *only* applies during the targeted intervention phase.
    if phase == 2
        desired = desired .* params.proph_strata;
    end

    if (sum(desired) <= MaxAVtoContacts)
        dProph_Given = desired;
    else
        if ~isfield(params,'Proph_prioritygroups')
            dProph_Given = desired * MaxAVtoContacts/sum(desired);
        else
            dProph_Given = priority_allocation(desired,params.Proph_prioritygroups,MaxAVtoContacts);
        end
    end

% Rate of prophylaxis to contacts of infected cases
dProph_I = dProph_Given .* DI_pcnt;
% Rate of effective prophylaxis to contacts of infected cases
% Efficacy of prophylaxis from near-patient diagnoses
np_efficacy = (params.e_dp .* params.frac_gp + (1 - params.frac_gp));
efficacy = zeros(n,1);
efficacy(~mask) = (np_efficacy(~mask) .* mild_presn_rate(~mask) + sev_presn_rate(~mask)) ./ presn_rate(~mask);
dProph_Eff = dProph_I .* efficacy;
% Return the parameters governing the prophylaxis of infected cases
epsilon = zeros(n,1);
all_contacts_made = (params.kappa'/365) * presn_rate;
epsilon(~mask) = dProph_Eff(~mask) ./ all_contacts_made(~mask);
proph = dProph_Given;
end

%%
% Allocate resource to priority groups and remainder (if any) equitably
function allocation = priority_allocation(desired,pgroups,available)

        % priority groups is an ordered list of the numerically specified
        % priority groups. Use as much of the limited resource on these
        % groups as possible, and the remainder in an equitable fashion.
        allocation = zeros(size(desired));
        remainder = available;
        mask = true(size(desired)); % For use in equitable allocation of remainder
        for i = 1:length(pgroups)
            indx = pgroups(i);
            mask(indx) = 0; % Update mask for use after this for loop
            allocation(indx) = min(remainder, desired(indx));
            remainder = remainder - allocation(indx);
        end
        if remainder>0
            % Allocate remaining capacity equitably
            allocation(mask) = desired(mask) * min(remainder/sum(desired(mask)), 1);
        end
end
