%%BURDEN   Calculates the healthcare burden for an AV strategy.
%   Calculates the burden on each healthcare setting (ICU, ward, ED, GP) that
%   result from the given AV usage strategy for a single pandemic scenario.
%
%   [result] = burden(scenario, strategy, hc);
%
%   SCENARIO is the pandemic scenario number (1-7).
%
%   STRATEGY is the AV usage strategy number (0-5).
%
%   HC is the healthcare system (see DEFAULT_HEALTHCARE for details).
%
%   RESULT is a structure that comprises the following fields:
%
%       * 'Scenario': the pandemic scenario number, as above.
%       * 'Strategy': the AV usage strategy number, as above.
%       * 'name': a 1xN cell array of burden-related statistics.
%       * 'obs': a 1xM cell array of observation names.
%       * 'value': a NxM matrix of calculated values.
%
%   Example:
%      %% Calculate the burden for a particular scenario and strategy.
%      >> res74 = burden(7, 4);
%      %% Print the mean number of deaths in the entire population.
%      >> res74.value(strcmp(res74.measure, 'Mean'), ...
%                     strcmp(res74.name, 'Death_TOTAL'))
%
% See also:
% BURDEN_TABLE
% DEFAULT_HEALTHCARE
function [result] = burden(scenario, strategy, hc)
    if (nargin == 2)
        hc = default_healthcare();
    end

    %% Calculate the burden-related outcomes.
    values = calc_outcomes(scenario, strategy, hc);
    cmp_baseline = strategy < 5;
    if (cmp_baseline)
        base_values = calc_outcomes(scenario, 5, hc);
    end

    %% TODO: compliance with mild treatment!?!
    %%       (will affect severe_to_mild flow)
    %% TODO: reduced efficacy of treatment due to delays in lab testing
    %%       (see diagnosis_regime.m for details)

    %% Define the name of each strata, statistic, and healthcare setting.
    strata_names = {'Children', 'Elderly', 'HighRisk', 'HCW', 'General'};
    obs_names = {'Median', '5% CI', '95% CI', 'Mean'};
    locn_names = {'ICU', 'Ward', 'ED', 'GP'};

    %% Functions for calculating statistics from data sets.
    qs = [0.50, 0.05, 0.95];
    stat_row = @(xs) [quantile(xs, qs) mean(xs)];
    stat_row2 = @(xs) [quantile(xs, qs)' mean(xs)'];

    %%
    %% Calculate summary statistics.
    %%
    %%   Per setting:
    %%     * Net and peak utilisation.
    %%     * Excess peak and duration.
    %%
    %%   Per strata:
    %%     * AV doses (treatment / prophylaxis / total).
    %%     * Net and peak hospitalisations (person days).
    %%     * Deaths.
    %%
    %%   Total:
    %%     * AV doses (treatment / prophylaxis / total).
    %%     * Net and peak hospitalisations (person days).
    %%     * Deaths.
    %%
    %%   In comparison to AV strategy 5 (no AVs):
    %%     * Hospitalisations avoided due to early treatment.
    %%     * Deaths averted.
    %%     * Reduction in net hospital burden.
    %%

    if (cmp_baseline)
        averted_deaths = base_values.total_deaths - values.total_deaths;
        averted_icu = base_values.want_icu - values.want_icu;
        averted_hosp = base_values.want_hosp - values.want_hosp;
        %% Relative risks of: presenting (i.e., CAP), hospitalisation, ICU
        %% admissions, and death.
        rr_CAP = values.CAP ./ base_values.CAP;
        rr_hosp = values.want_hosp ./ base_values.want_hosp;
        rr_icu = values.want_icu ./ base_values.want_icu;
        rr_death = values.total_deaths ./ base_values.total_deaths;
    else
        averted_deaths = zeros(size(values.total_deaths));
        averted_icu = zeros(size(values.total_icu));
        averted_hosp = zeros(size(values.total_ward));
        rr_CAP = zeros(size(values.CAP));
        rr_hosp = zeros(size(values.want_hosp));
        rr_icu = zeros(size(values.want_icu));
        rr_death = zeros(size(values.total_deaths));
    end

    stats = [stat_row(values.total_icu); ...
             stat_row(values.total_ward); ...
             stat_row(values.total_ed); ...
             stat_row(values.total_gp); ...
             stat_row(values.peak_icu); ...
             stat_row(values.peak_ward); ...
             stat_row(values.peak_ed); ...
             stat_row(values.peak_gp); ...
             stat_row(values.icu_excess_durn); ...
             stat_row(values.ward_excess_durn); ...
             stat_row(values.ed_excess_durn); ...
             stat_row(values.gp_excess_durn); ...
             stat_row(values.peak_icu_excess); ...
             stat_row(values.peak_ward_excess); ...
             stat_row(values.peak_ed_excess); ...
             stat_row(values.peak_gp_excess); ...
             stat_row(values.total_deaths); ...
             stat_row(averted_deaths); ...
             stat_row(averted_hosp); ...
             stat_row(averted_icu); ...
             stat_row2(values.strata_deaths); ...
             stat_row2(values.strata_icu); ...
             stat_row2(values.strata_ward); ...
             stat_row2(values.strata_ed); ...
             stat_row2(values.strata_gp); ...
             stat_row2(values.strata_excess_icu); ...
             stat_row2(values.strata_excess_ward); ...
             stat_row2(values.strata_excess_ed); ...
             stat_row2(values.strata_excess_gp); ...
             stat_row(values.want_icu); ...
             stat_row(values.want_ward); ...
             stat_row2(values.strata_rx); ...
             stat_row2(values.strata_rx_mld); ...
             stat_row2(values.strata_rx_early); ...
             stat_row2(values.strata_rx_hosp); ...
             stat_row2(values.strata_norx_hosp); ...
             stat_row2(values.strata_sev2mld); ...
             stat_row(values.total_sev2mld); ...
             stat_row2(values.CAP); ...
             stat_row2(values.av_doses); ...
             stat_row(values.av_4wk_treat); ...
             stat_row(values.av_4wk_total); ...
             stat_row2(rr_CAP); ...
             stat_row(rr_hosp); ...
             stat_row(rr_icu); ...
             stat_row(rr_death); ...
            ];

    num_rows = size(stats, 1);
    stats = [repmat([scenario strategy], num_rows, 1) stats];
    stat_names = [strcat('TotalCases_', locn_names) ...
                  strcat('PeakCases_', locn_names) ...
                  strcat('ExcessDurn_', locn_names) ...
                  strcat('ExcessPeak_', locn_names) ...
                  'Death_Total' ...
                  'Death_Averted' ...
                  'Hosp_Averted' ...
                  'ICU_Averted' ...
                  strcat('Death_', strata_names) ...
                  strcat('AdmitICU_', strata_names) ...
                  strcat('AdmitWard_', strata_names) ...
                  strcat('AdmitED_', strata_names) ...
                  strcat('AdmitGP_', strata_names) ...
                  strcat('ExcessICU_', strata_names) ...
                  strcat('ExcessWard_', strata_names) ...
                  strcat('ExcessED_', strata_names) ...
                  strcat('ExcessGP_', strata_names) ...
                  'WantICU' ...
                  'WantWard' ...
                  strcat('AVTreat_', strata_names) ...
                  strcat('AVTreatMild_', strata_names) ...
                  strcat('AVTreatEarly_', strata_names) ...
                  strcat('AVTreatHosp_', strata_names) ...
                  strcat('HospNoRx_', strata_names) ...
                  strcat('SevToMild_', strata_names) ...
                  'SevToMild_Total' ...
                  strcat('CAP_', strata_names) ...
                  strcat('AV_', {'Net', 'Proph', 'Treat', ...
                                 'Severe', 'Mild', 'NonFlu'}) ...
                  'AV_4weekTreat' ...
                  'AV_4weekTotal' ...
                  strcat('RR_CAP', strata_names) ...
                  'RR_Hosp' ...
                  'RR_ICU' ...
                  'RR_Death'
                 ];

    result = struct('Scenario', scenario, ...
                    'Strategy', strategy, ...
                    'name', {stat_names}, ...
                    'obs', {['scenario' 'strategy' obs_names]}, ...
                    'value', {stats});
end

function values = calc_outcomes(scenario, strategy, hc)
    exp_name = sprintf('Scenario%d_AVstrategy%d', scenario, strategy);
    data = most_recent(exp_name);

    stats = data.lhs_data.stats;
    layout = data.lhs_data.stat_layout;
    fixed_params = data.lhs_data.unsampled_params;
    sampled_params = data.lhs_data.parameter_samples;

    %% Extract the population size for each strata.
    N_strata = fixed_params.pi * fixed_params.N;

    %% Define some frequently-used dimension sizes.
    num_strata = length(N_strata);
    num_days = layout.DailyIncP(3);

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%
    %% Burden on each healthcare setting, deaths, etc.
    %%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    %%
    %% Step 0: Calculate the daily incidence of presenting cases.
    %%

    di_net = get_daily_incidence(data);

    %%
    %% Step 1: Calculate the limits on daily AV treatment doses and identify
    %%         which strata (if any) receive treatment for mild cases.
    %%

    [max_treat to_treat_mild] = av_treatment_limits(data);

    %%
    %% Step 2: Determine the daily incidence of presentations that, in the
    %%         absence of treatment, require hospitalisation.
    %%

    [p_sev p_mld] = get_daily_presns(data, di_net);
    p_mld_GP = hc.mild_to_GP * p_mld;
    p_mld_ED = hc.mild_to_ED * p_mld;

    %%
    %% Step 3: Determine what number of cases will avoid hospitalisation as a
    %%         result of receiving early treatment at a GP clinic (if any).
    %%
    %% How to respect MaxAVtoTreat and doses for hospitalised cases?
    %%
    %% NOTE: In only 4% of the Scenario 7 simulations does daily *incidence*
    %%       exceed the daily AV distribution limit of 1e4.
    %%       Clearly, the hospital capacities will be a much greater limit on
    %%       clinical burden and deaths.
    %%       So, for now, assume we have enough AV doses to provide early
    %%       treatment to all potentially-severe cases, and use the remaining
    %%       distribution to treat (a) hospitalised cases; and (b) mild cases.
    %%
    p_rx_early = hc.early_rx_sev_propn * (to_treat_mild .* p_sev);
    p_sev_to_mld = bsxfun(@times, p_rx_early, hc.early_rx_sev_to_mild);
    p_hosp = p_sev - p_sev_to_mld;
    cap_treat = max_treat;
    for s = 1:num_strata
        cap_treat = cap_treat - squeeze(p_rx_early(:, s, :));
    end
    %% Ensure the remaining capacity remains non-negative.
    cap_treat = max(cap_treat, 0);

    %%
    %% Step 4: Treatment of cases that require hospitalisation (regardless of
    %%         whether or not there are available beds) and mild cases that
    %%         present to EDs and GPs (regardless of consultation capacity).
    %%

    %% Determine what coverage we can provide, accounting for priorities.
    p_hosp_rx = zeros(size(p_hosp));
    for s = hc.priority
        want_rx = squeeze(p_hosp(:, s, :));
        p_hosp_rx(:, s, :) = min(want_rx, cap_treat);
        cap_treat = cap_treat - squeeze(p_hosp_rx(:, s, :));
    end
    p_hosp_norx = p_hosp - p_hosp_rx;

    p_mld_rx = zeros(size(p_mld));
    for s = hc.priority
        want_rx = squeeze(p_mld(:, s, :) .* to_treat_mild(:, s, :));
        p_mld_rx(:, s, :) = min(want_rx, cap_treat);
        cap_treat = cap_treat - squeeze(p_mld_rx(:, s, :));
    end
    p_mld_norx = p_mld - p_mld_rx;

    %% Count the daily AV treatment doses for each strata.
    p_rx = p_hosp_rx + p_mld_rx + p_rx_early;

    %%
    %% Step 5: Deaths (from cases that required ICU care, regardless of
    %%         actual availability).
    %%
    p_icu_rx = bsxfun(@times, p_hosp_rx, hc.hosp_rx_to_ICU);
    p_ward_rx = p_hosp_rx - p_icu_rx;
    p_icu_norx = bsxfun(@times, p_hosp_norx, hc.hosp_to_ICU);
    p_ward_norx = p_hosp_norx - p_icu_norx;
    daily_deaths = bsxfun(@times, p_icu_rx, hc.hosp_rx_ICU_RoD) ...
                   + bsxfun(@times, p_icu_norx, hc.hosp_ICU_RoD);

    %%
    %% Step 6: Presentation locations, ignoring actual capacities.
    %%
    p_icu = p_icu_rx + p_icu_norx;
    p_ward = p_ward_rx + p_ward_norx;
    p_gp = bsxfun(@times, p_mld, hc.mild_to_GP);
    p_ed = p_mld - p_gp;
    %% Note: we've already accounted for p_rx_early ...

    %%
    %% Step 7: Calculate incidence at each location, ignoring capacities.
    %%
    inc_icu = p_icu;
    for d = 1:hc.LoS_ICU - 1
        presns = p_icu(:, :, 1:end - d);
        ix = 1 + d;
        inc_icu(:, :, ix:end) = inc_icu(:, :, ix:end) + presns;
    end
    inc_ward = p_ward;
    for d = 1:hc.LoS_Ward - 1
        presns = p_ward(:, :, 1:end - d);
        ix = 1 + d;
        inc_ward(:, :, ix:end) = inc_ward(:, :, ix:end) + presns;
    end

    %%
    %% Step 8: Calculate burden at each location, accounting for capacities.
    %%
    avail_icu = repmat(hc.cap_ICU, [data.lhs_data.samples num_days]);
    admit_icu = zeros(size(inc_icu));
    for s = hc.priority
        new_icu = min(squeeze(inc_icu(:, s, :)), avail_icu);
        avail_icu = avail_icu - new_icu;
        admit_icu(:, s, :) = new_icu;
    end
    %% Defer the excess burden to general wards.
    excess_icu = inc_icu - admit_icu;
    inc_ward = inc_ward + excess_icu;

    avail_ward = repmat(hc.cap_Ward, [data.lhs_data.samples num_days]);
    admit_ward = zeros(size(inc_ward));
    for s = hc.priority
        new_ward = min(squeeze(inc_ward(:, s, :)), avail_ward);
        avail_ward = avail_ward - new_ward;
        admit_ward(:, s, :) = new_ward;
    end
    %% Defer the excess burden to emergency departments.
    %% Note: we are not converting incidence back to prevalence!
    %%       These cases will return for care every day for 5 days.
    excess_ward = inc_ward - admit_ward;
    p_ed = p_ed + excess_ward;

    avail_ed = repmat(hc.cap_ED, [data.lhs_data.samples num_days]);
    admit_ed = zeros(size(p_ed));
    for s = hc.priority
        new_ed = min(squeeze(p_ed(:, s, :)), avail_ed);
        avail_ed = avail_ed - new_ed;
        admit_ed(:, s, :) = new_ed;
    end
    %% Defer the excess burden to GP clinics.
    excess_ed = p_ed - admit_ed;
    p_gp = p_gp + excess_ed;

    avail_gp = repmat(hc.cap_GP, [data.lhs_data.samples num_days]);
    %% Account for the potentially-severe cases that presented to GPs.
    avail_gp = avail_gp - squeeze(sum(p_rx_early, 2));
    admit_gp = zeros(size(p_gp));
    for s = hc.priority
        new_gp = min(squeeze(p_gp(:, s, :)), avail_gp);
        avail_gp = avail_gp - new_gp;
        admit_gp(:, s, :) = new_gp;
    end
    %% Count the excess burden on GP clinics.
    excess_gp = p_gp - admit_gp;

    %%
    %% Step 9: Calculate and return the relevant data series.
    %%
    values = struct();
    %% Net treatment, prophylaxis and total AV doses, then treatment doses for
    %% severe cases, for mild cases, and for non-flu (mild ILI) cases.
    %% display(layout)
    values.av_doses = squeeze(stats(1, :, [66 65 64 67:69]));
    values.av_4wk_treat = squeeze(stats(1, :, 71));
    values.av_4wk_total = squeeze(stats(1, :, 71) + stats(1, :, 73));
    values.strata_deaths = sum(daily_deaths, 3);
    values.total_deaths = sum(values.strata_deaths, 2);
    values.strata_excess_icu = sum(excess_icu, 3);
    values.strata_excess_ward = sum(excess_ward, 3);
    values.strata_excess_gp = sum(excess_ed, 3);
    values.strata_excess_ed = sum(excess_ed, 3);
    values.daily_icu_excess = squeeze(sum(excess_icu, 2));
    values.daily_ward_excess = squeeze(sum(excess_ward, 2));
    values.daily_ed_excess = squeeze(sum(excess_ed, 2));
    values.daily_gp_excess = squeeze(sum(excess_gp, 2));
    values.peak_icu_excess = max(values.daily_icu_excess, [], 2);
    values.peak_ward_excess = max(values.daily_ward_excess, [], 2);
    values.peak_ed_excess = max(values.daily_ed_excess, [], 2);
    values.peak_gp_excess = max(values.daily_gp_excess, [], 2);
    values.icu_excess_durn = sum(values.daily_icu_excess > 0, 2);
    values.ward_excess_durn = sum(values.daily_ward_excess > 0, 2);
    values.ed_excess_durn = sum(values.daily_ed_excess > 0, 2);
    values.gp_excess_durn = sum(values.daily_gp_excess > 0, 2);
    values.daily_icu = hc.cap_ICU - avail_icu;
    values.daily_ward = hc.cap_Ward - avail_ward;
    values.daily_ed = hc.cap_ED - avail_ed;
    values.daily_gp = hc.cap_GP - avail_gp;
    values.peak_icu = max(values.daily_icu, [], 2);
    values.peak_ward = max(values.daily_ward, [], 2);
    values.peak_ed = max(values.daily_ed, [], 2);
    values.peak_gp = max(values.daily_gp, [], 2);
    values.strata_icu = sum(admit_icu, 3);
    values.strata_ward = sum(admit_ward, 3);
    values.strata_ed = sum(admit_ed, 3);
    values.strata_gp = sum(admit_gp, 3);
    values.total_icu = sum(values.strata_icu, 2);
    values.total_ward = sum(values.strata_ward, 2);
    values.total_ed = sum(values.strata_ed, 2);
    values.total_gp = sum(values.strata_gp, 2);
    values.want_hosp = sum(sum(p_hosp, 3), 2);
    values.want_icu = sum(sum(p_icu, 3), 2);
    values.want_ward = sum(sum(p_ward, 3), 2);
    values.strata_rx = sum(p_rx, 3);
    values.strata_rx_mld = sum(p_mld_rx, 3);
    values.strata_rx_early = sum(p_rx_early, 3);
    values.strata_rx_hosp = sum(p_hosp_rx, 3);
    values.strata_norx_hosp = sum(p_hosp_norx, 3);
    values.strata_sev2mld = sum(p_sev_to_mld, 3);
    values.total_sev2mld = sum(values.strata_sev2mld, 2);
    values.CAP = bsxfun(@rdivide, sum(di_net, 3), N_strata');
end


%%
%% Step 0: Calculate the daily incidence of presenting cases.
%%
function [di_net] = get_daily_incidence(data)
    stats = data.lhs_data.stats;
    layout = data.lhs_data.stat_layout;
    stat_names = fieldnames(layout);
    ix_di = 1;
    ix_dip = 1;
    ix_end = 1;
    for ix = 1:length(stat_names)
        name = stat_names{ix};
        stat_dims = layout.(name);
        stat_sz = stat_dims(1);

        if (strcmp(name, 'DailyIncNP'))
            ix_dip = ix_dip + stat_sz;
            ix_end = ix_end + stat_sz;
        elseif (strcmp(name, 'DailyIncP'))
            ix_end = ix_end + stat_sz;
            break
        else
            ix_di = ix_di + stat_sz;
            ix_dip = ix_dip + stat_sz;
            ix_end = ix_end + stat_sz;
        end
    end
    %% [ix_di ix_dip ix_end]

    %% Extract the daily incidence for each strata, distinguishing between
    %% cases that have and have not received post-exposure prophylaxis.
    di_np = stats(1, :, ix_di:ix_dip - 1);
    di_np = reshape(di_np, [data.lhs_data.samples layout.DailyIncNP(2:3)]);
    di_wp = stats(1, :, ix_dip:ix_end - 1);
    di_wp = reshape(di_wp, [data.lhs_data.samples layout.DailyIncP(2:3)]);
    %% Calculate the net daily incidence for each strata.
    di_net = di_np + di_wp;

end

%%
%% Step 1: Calculate the limits on daily AV treatment doses and identify
%%         which strata (if any) receive treatment for mild cases.
%%
function [max_treat to_treat_mild] = av_treatment_limits(data)
    stats = data.lhs_data.stats;
    fixed_params = data.lhs_data.unsampled_params;
    sampled_params = data.lhs_data.parameter_samples;
    num_strata = data.lhs_data.stat_layout.DailyIncP(2);
    num_days = data.lhs_data.stat_layout.DailyIncP(3);

    %%
    %% Step 1: Determine the timing of the Control and Target phases.
    %%

    %% Determine when the Control and Target phases began.
    Start_Control_Day = stats(1, :, 61);
    Start_Target_Day = floor(Start_Control_Day + 28);
    %% Determine the indices into the daily incidence arrays that identify the
    %% start of the Target phase; the "+ 1" is necessary because these arrays
    %% start with day 0 at index 1.
    ix_Target = Start_Target_Day + 1;
    %% Similar argument for the start of the Control phase.
    ix_Control = floor(Start_Control_Day) + 1;

    %% Determine the maximum number of AV treatment doses that can be
    %% delivered daily during each of the two phases.
    Control_MaxTreat = sampled_params.MaxAVtoTreat_Ph1;
    Target_MaxTreat = sampled_params.MaxAVtoTreat_Ph2;
    if (isfield(sampled_params, 'O'))
        %% Important: for AV strategy 5 (no AV stockpile), the stockpile size
        %% was set to zero, but the MaxAVtoTreat parameters were not changed.
        %% Without the following two lines, this function will provide
        %% treatment to presenting cases.
        Control_MaxTreat = min(Control_MaxTreat, sampled_params.O);
        Target_MaxTreat = min(Target_MaxTreat, sampled_params.O);
    end

    %%
    %% Step 2: Calculate the daily limit on AV treatment doses.
    %%

    max_treat = zeros([data.lhs_data.samples num_days]);
    for d = 1:num_days
        mask = d >= ix_Control;
        if (any(mask))
            max_treat(mask, d) = Control_MaxTreat(mask)';
        end
    end
    for d = 1:num_days
        mask = d >= ix_Target;
        if (any(mask))
            max_treat(mask, d) = Target_MaxTreat(mask)';
        end
    end

    %%
    %% Step 3: Identify which strata receive AV treatment for mild cases in
    %%         the Target phase.
    %%
    if (isfield(sampled_params, 'treat_strata'))
        treat_strata = squeeze(sampled_params.treat_strata);
    else
        %% Note: we need to resize this vector.
        treat_strata = repmat(fixed_params.treat_strata', ...
                              data.lhs_data.samples, 1);
    end
    %% to_treat_mild = zeros(size(di_net));
    to_treat_mild = zeros([data.lhs_data.samples num_strata num_days]);
    for d = 1:num_days
        mask = d >= ix_Control & Control_MaxTreat > 0;
        if (any(mask))
            to_treat_mild(mask, :, d) = 1;
        end
    end
    for d = 1:num_days
        mask = d >= ix_Target & Target_MaxTreat > 0;
        if (any(mask))
            to_treat_mild(mask, :, d) = treat_strata(mask, :);
        end
    end

end

%%
%% Step 2: Determine the daily incidence of presentations that, in the
%%         absence of treatment, require hospitalisation.
%%
function [p_sev p_mld] = get_daily_presns(data, di_net)
    eta = squeeze(data.lhs_data.parameter_samples.eta);
    eta = bsxfun(@times, eta, data.lhs_data.unsampled_params.eta_relscale');
    p_sev = bsxfun(@times, eta, di_net);
    p_mld = bsxfun(@times, 1 - eta, di_net);
end
