%MOST_RECENT   Loads the most recent results for a given experiment.
%   Loads the most recent results (.mat file) for the experiment whose name
%   matches the specified pattern. The optional parameter |nth_last| can be
%   used to open a less-recent result.
%
%   [data, filename, fullpath] = most_recent(experiment, nth_last);
%
%   EXPERIMENT is the name of the LHS experiment.
%
%   NTH_LAST (optionally) specifies to load the nth-most recent results.
%
%   DATA is the contents of the .mat file (typically a single structure
%   containing the details of a LHS experiment).
%
%   FILENAME is the name of the .mat file from which the results were taken,
%   without the .mat extension.
%
%   FULLPATH is the absolute path of the .mat file.
%
%   Example:
%      % Loads the third-last run of the experiment 'some_exp'
%      >> data = most_recent('some_exp', 3);
%
%See also filename, filename_timestamped
function [result] = get_daily_inc(scenario, strategy)

    %% TODO:
    %%
    %% 1. Store all health-care parameters (anything that we want to allow a
    %%    user to modify) in a single struct and provide a default instance.
    %%
    %%    (Also see http://stackoverflow.com/a/2776238 for ideas).
    %%
    %% 2. Figure out how best to print/save these data.
    %%
    %%    NOTE: The current structure needs improvement!
    %%
    %%    summ_tbl = cell(7, 6);
    %%    for S = 1:7
    %%        for A = 0:5
    %%            summ_tbl{S, A + 1} = get_daily_inc(S, A);
    %%        end
    %%    end
    %%    save('summary_table.mat', 'summ_tbl', '-v7.3');
    %%
    %% 3. Separate the calculations from the LHS data structure so that, e.g.,
    %%    we can calculate them for a single simulation by providing a subset
    %%    of the lhs_data.stats array.
    %%
    %% 4. Calculate, e.g., deaths avoided per AV dose, the net burden at each
    %%    setting?
    %%

    exp_name = sprintf('Scenario%d_AVstrategy%d', scenario, strategy);
    data = most_recent(exp_name);
    stats = data.lhs_data.stats;
    layout = data.lhs_data.stat_layout;
    fixed_params = data.lhs_data.unsampled_params;
    sampled_params = data.lhs_data.parameter_samples;

    %% Extract the population size for each strata.
    N_strata = fixed_params.pi * fixed_params.N;

    %% Define the name of each strata, statistic, and healthcare setting.
    strata_names = {'Children', 'Elderly', 'HighRisk', 'HCW', 'General'};
    obs_names = {'Median', '5% CI', '95% CI', 'Mean'};
    locn_names = {'ICU', 'Ward', 'ED', 'GP'};

    %% Calculate AV usage and 2 week/4 week statistics.
    av_total = stats(1, :, 66);
    av_treat = stats(1, :, 64);
    av_proph = stats(1, :, 65);
    av_severe = stats(1, :, 67);
    av_mild = stats(1, :, 68);
    av_nonflu = stats(1, :, 69);
    av_2wk_tr = stats(1, :, 70);
    av_4wk_tr = stats(1, :, 71);
    av_2wk_pr = stats(1, :, 72);
    av_4wk_pr = stats(1, :, 73);
    cinc_2wk = stats(1, :, 342:346);
    cinc_4wk = stats(1, :, 347:351);

    %% Summarise the AV usage and 2 week/4 week statistics.
    function [out] = summarise(ixs, names)
        series = squeeze(stats(1, :, ixs));
        vals = [median(series); quantile(series, [0.05]); ...
                quantile(series, [0.95]); mean(series); ];
        out = dataset([vals, names], 'obsnames', obs_names);
    end
    %% Net treatment, prophylaxis and total AV doses, then treatment doses for
    %% severe cases, for mild cases, and for non-flu (mild ILI) cases.
    av_summary = summarise([66 65 64 67:69], ...
                           strcat('AV_', {'Net', 'Proph', 'Treat', ...
                                          'Severe', 'Mild', 'NonFlu'}));
    %% Treatment doses at 2 & 4 weeks, prophylaxis doses at 2 & 4 weeks.
    av_wk_summary = summarise(70:73, {'Treat_2weeks', 'Treat_4weeks', ...
                                      'Proph_2weeks', 'Proph_4weeks'});
    ci_2wk_summary = summarise(342:346, strcat('CI2wk_', strata_names));
    ci_4wk_summary = summarise(347:351, strcat('CI4wk_', strata_names));

    stat_names = fieldnames(layout);
    ix_di = 1;
    ix_dip = 1;
    ix_end = 1;
    for ix = 1:length(stat_names)
        name = stat_names{ix};
        stat_dims = layout.(name);
        stat_sz = stat_dims(1);

        if (strcmp(name, 'DailyIncNP'))
            ix_dip = ix_dip + stat_sz;
            ix_end = ix_end + stat_sz;
        elseif (strcmp(name, 'DailyIncP'))
            ix_end = ix_end + stat_sz;
            break
        else
            ix_di = ix_di + stat_sz;
            ix_dip = ix_dip + stat_sz;
            ix_end = ix_end + stat_sz;
        end
    end
    %% [ix_di ix_dip ix_end]

    %% Extract the daily incidence for each strata, distinguishing between
    %% cases that have and have not received post-exposure prophylaxis.
    di_np = stats(1, :, ix_di:ix_dip - 1);
    di_np = reshape(di_np, [data.lhs_data.samples layout.DailyIncNP(2:3)]);
    di_wp = stats(1, :, ix_dip:ix_end - 1);
    di_wp = reshape(di_wp, [data.lhs_data.samples layout.DailyIncP(2:3)]);
    di_net = di_np + di_wp;

    %% Calculate the clinical attack proportions.
    cuminc = sum(di_net, 3);
    CAP = bsxfun(@rdivide, cuminc, N_strata');
    CAP_mean = mean(CAP);
    CAP_median = median(CAP);
    CAP_qtls = quantile(CAP, [0.05 0.95]);
    CAP_summary = dataset([[CAP_median; CAP_qtls; CAP_mean], ...
                           strcat('CAP_', strata_names)], ...
                          'obsnames', obs_names);

    %%
    %% TODO: Antiviral effects -- FUDGE FACTORS FOR NOW -- store in a STRUCT.
    %%
    proph_sev_to_mild_NR = 0.16;
    proph_reduce_death_NR = 0.5;
    proph_sev_to_mild_HR = 0.52;
    proph_reduce_death_HR = 0.25;
    treat_sev_to_mild_NR = 0.16;
    treat_reduce_death_NR = 0.5;
    treat_sev_to_mild_HR = 0.52;
    treat_reduce_death_HR = 0.25;

    %% Collect the effects into per-strata vectors.
    proph_sev_to_mild = [proph_sev_to_mild_NR, proph_sev_to_mild_NR, ...
                         proph_sev_to_mild_HR, ...
                         proph_sev_to_mild_NR, proph_sev_to_mild_NR];
    proph_reduce_death = [proph_reduce_death_NR, proph_reduce_death_NR, ...
                          proph_reduce_death_HR, ...
                          proph_reduce_death_NR, proph_reduce_death_NR];
    treat_sev_to_mild = [treat_sev_to_mild_NR, treat_sev_to_mild_NR, ...
                         treat_sev_to_mild_HR, ...
                         treat_sev_to_mild_NR, treat_sev_to_mild_NR];
    treat_reduce_death = [treat_reduce_death_NR, treat_reduce_death_NR, ...
                          treat_reduce_death_HR, ...
                          treat_reduce_death_NR, treat_reduce_death_NR];

    proph_sev_to_mild = repmat(proph_sev_to_mild', 1, 366);
    proph_reduce_death = repmat(proph_reduce_death', 1, 366);
    treat_sev_to_mild = repmat(treat_sev_to_mild', 1, 366);
    treat_reduce_death = repmat(treat_reduce_death', 1, 366);
    proph_sev_to_mild = shiftdim(proph_sev_to_mild, -1);
    proph_reduce_death = shiftdim(proph_reduce_death, -1);
    treat_sev_to_mild = shiftdim(treat_sev_to_mild, -1);
    treat_reduce_death = shiftdim(treat_reduce_death, -1);

    %%
    %% Calculate the daily burden for each health-care setting.
    %%

    %% Determine the capacity constraints for each health-care setting.
    %% TODO: store in a STRUCT.
    cap_gp = fixed_params.cap_gp;
    cap_ed = fixed_params.cap_ed;
    cap_ward = fixed_params.cap_ward;
    cap_icu = fixed_params.cap_icu;
    %% Determine the ratio of presentations to each setting.
    mild_to_gp = shiftdim(repmat(0.8, 5, 366), -1);
    mild_to_ed = 1 - mild_to_gp;
    severe_to_icu_NR = 0.125;
    severe_to_icu_HR = severe_to_icu_NR * 2;
    severe_to_icu = [severe_to_icu_NR, severe_to_icu_NR, ...
                     severe_to_icu_HR, ...
                     severe_to_icu_NR, severe_to_icu_NR];
    severe_to_icu = shiftdim(repmat(severe_to_icu', 1, 366), -1);
    severe_to_ward = 1 - severe_to_icu;
    %% Probabilities of death.
    pr_die_ward = shiftdim(repmat(0.05, 5, 366), -1);
    pr_die_icu = shiftdim(repmat([0.05 0.05 0.075 0.05 0.05]', 1, 366), -1);
    %% Define the mean length of stay for hospitalised cases.
    los_ward = 5;
    los_icu = 10;

    %% Determine when the Control and Target phases began.
    Start_Control_Day = stats(1, :, 61);
    Start_Target_Day = floor(Start_Control_Day + 28);
    %% Determine the indices into the daily incidence arrays that identify the
    %% start of the Target phase; the "+ 1" is necessary because these arrays
    %% start with day 0 at index 1.
    ix_Target = Start_Target_Day + 1;
    %% Similar argument for the start of the Control phase.
    ix_Control = floor(Start_Control_Day) + 1;

    %% Determine the maximum number of AV treatment doses that can be
    %% delivered daily during each of the two phases.
    Control_MaxTreat = sampled_params.MaxAVtoTreat_Ph1;
    Target_MaxTreat = sampled_params.MaxAVtoTreat_Ph2;
    if (isfield(sampled_params, 'O'))
        %% Important: for AV strategy 5 (no AV stockpile), the stockpile size
        %% was set to zero, but the MaxAVtoTreat parameters were not changed.
        %% Without the following two lines, this function will provide
        %% treatment to presenting cases.
        Control_MaxTreat = min(Control_MaxTreat, sampled_params.O);
        Target_MaxTreat = min(Target_MaxTreat, sampled_params.O);
    end

    %% Calculate the daily limit on AV treatment doses.
    %% max_treat = zeros(size(di_net));
    max_treat = zeros([data.lhs_data.samples 366]);
    for d = 1:366
        mask = d >= ix_Control;
        if (any(mask))
            max_treat(mask, d) = Control_MaxTreat(mask)';
        end
    end
    for d = 1:366
        mask = d >= ix_Target;
        if (any(mask))
            max_treat(mask, d) = Target_MaxTreat(mask)';
        end
    end

    %% Determine the strategies for treating mild cases.
    if (isfield(sampled_params, 'treat_strata'))
        treat_strata = squeeze(sampled_params.treat_strata);
    else
        %% Note: we need to resize this vector.
        treat_strata = repmat(fixed_params.treat_strata', ...
                              data.lhs_data.samples, 1);
    end

    %% Identify which strata receive AV treatment for mild cases.
    to_treat_mild = zeros(size(di_net));
    for d = 1:366
        mask = d >= ix_Control;
        if (any(mask))
            to_treat_mild(mask, :, d) = 1;
        end
    end
    for d = 1:366
        mask = d >= ix_Target;
        if (any(mask))
            to_treat_mild(mask, :, d) = treat_strata(mask, :);
        end
    end

    %% Determine the fraction of presentations that, in the absence of
    %% treatment or prophylaxis, are severe.
    eta = squeeze(sampled_params.eta);
    eta = bsxfun(@times, eta, fixed_params.eta_relscale');

    %% Calculate the daily incidence of mild and severe presentations, without
    %% accounting for the effects of any treatment or prophylaxis.
    p_np_sev = bsxfun(@times, eta, di_np);
    p_np_mld = bsxfun(@times, 1 - eta, di_np);
    p_wp_sev = bsxfun(@times, eta, di_wp);
    p_wp_mld = bsxfun(@times, 1 - eta, di_wp);

    %% Calculate the effects of prophylaxis on presentations.
    p_wp_sev_to_mild_by_proph = bsxfun(@times, p_wp_sev, proph_sev_to_mild);
    p_wp_mld = p_wp_mld + p_wp_sev_to_mild_by_proph;
    p_wp_sev = p_wp_sev - p_wp_sev_to_mild_by_proph;

    %% Calculate the daily incidence of mild and severe presentations, without
    %% accounting for the effects of AV treatment.
    p_mld = p_wp_mld + p_np_mld;
    p_sev = p_wp_sev + p_np_sev;

    %% The treatment priority: HCW, High-Risk, Children, Elderly, General.
    priority = [4, 3, 1, 2, 5];

    %% Determine which severe cases receive AV treatment.
    sev_tr = zeros(size(p_sev));
    for s = priority
        pres = squeeze(p_sev(:, s, :));
        sev_tr(:, s, :) = min(pres, max_treat);
        max_treat = max_treat - squeeze(sev_tr(:, s, :));
    end
    sev_notr = p_sev - sev_tr;

    %% Determine which mild cases receive AV treatment.
    mld_tr = zeros(size(p_mld));
    for s = priority
        %% Treat mild cases according to the "to_treat_mild" matrix.
        pres = squeeze(p_mld(:, s, :) .* to_treat_mild(:, s, :));
        mld_tr(:, s, :) = min(pres, max_treat);
        max_treat = max_treat - squeeze(mld_tr(:, s, :));
    end
    mld_notr = p_mld - mld_tr;

    %% Calculate the effect of treatment on presentations.
    p_sev_to_mild = bsxfun(@times, sev_tr, treat_sev_to_mild);
    p_mld = p_mld + p_sev_to_mild;
    p_sev = p_sev - p_sev_to_mild;

    %% Now that we have the *ultimate* severity for each presentation, we need
    %% to calculate the ultimate *location* and then account for capacities.
    %%
    %% Note: we can now calculate deaths from the severe cases.
    %% We assume that 5% of hospitalised cases result in death, and that
    %% high-risk patients admitted to ICUs have a 50% higher relative risk.
    %% We can presumably ignore the capacity constraints for these statistics,
    %% on the grounds that the ultimate outcome is already decided.
    p_icu = bsxfun(@times, p_sev, severe_to_icu);
    p_ward = bsxfun(@times, p_sev, severe_to_ward);
    p_gp = bsxfun(@times, p_mld, mild_to_gp);
    p_ed = bsxfun(@times, p_mld, mild_to_ed);
    %% Determine the provision of prophylaxis and treatment across the severe
    %% cases, assuming that treatment is given independently of whether or not
    %% a patient previously received prophylaxis.
    sev_frac_wp = zeros(size(p_wp_sev));
    is_nonz = p_sev > 0;
    sev_frac_wp(is_nonz) = p_wp_sev(is_nonz) ./ (p_wp_sev(is_nonz) + p_np_sev(is_nonz));
    sev_frac_np = 1 - sev_frac_wp;
    %% Identify the severe cases that received treatment and did not become
    %% mild cases as a result.
    sev_tr_wp = (sev_tr - p_sev_to_mild) .* sev_frac_wp;
    sev_tr_np = (sev_tr - p_sev_to_mild) .* sev_frac_np;
    %% Identify the severe cases that did not receive treatment.
    sev_notr_wp = sev_notr .* sev_frac_wp;
    sev_notr_np = sev_notr .* sev_frac_np;
    %% Calculate the risk of death for both ICU and general ward admissions.
    pr_die = severe_to_icu .* pr_die_icu + severe_to_ward .* pr_die_ward;
    %% Account for the protective effects of prophylaxis and treatment.
    pr_die_pr = pr_die .* (1 - proph_reduce_death);
    pr_die_tr = pr_die .* (1 - treat_reduce_death);
    %% Assume the combined effect is simply the best of the two.
    pr_die_both = pr_die .* (1 - max(proph_reduce_death, treat_reduce_death));
    daily_deaths = bsxfun(@times, pr_die, sev_notr_np) ...
                   + bsxfun(@times, pr_die_tr, sev_tr_np) ...
                   + bsxfun(@times, pr_die_pr, sev_notr_wp) ...
                   + bsxfun(@times, pr_die_both, sev_tr_wp);
    %% Calculate the total deaths across each strata and the whole population.
    strata_deaths = sum(daily_deaths, 3);
    total_deaths = sum(strata_deaths, 2);
    all_deaths = [strata_deaths, total_deaths];

    death_median = median(all_deaths);
    death_mean = mean(all_deaths);
    death_qtls = quantile(all_deaths, [0.05 0.95]);
    death_summary = dataset([[death_median; death_qtls; death_mean], ...
                             strcat('Death_', [strata_names, 'TOTAL'])], ...
                            'obsnames', obs_names);

    %% Convert presentations to incidence, accounting for length of stay.
    inc_icu = p_icu;
    for d = 1:los_icu - 1
        presns = p_icu(:, :, 1:end - d);
        ix = 1 + d;
        inc_icu(:, :, ix:end) = inc_icu(:, :, ix:end) + presns;
    end
    inc_ward = p_ward;
    for d = 1:los_ward - 1
        presns = p_ward(:, :, 1:end - d);
        ix = 1 + d;
        inc_ward(:, :, ix:end) = inc_ward(:, :, ix:end) + presns;
    end
    inc_gp = p_gp;
    inc_ed = p_ed;

    %% Now account for diagnostic and bed capacities.
    avail_icu = repmat(cap_icu, [data.lhs_data.samples 366]);
    admit_to_icu = zeros(size(inc_icu));
    for s = priority
        new_icu = min(squeeze(inc_icu(:, s, :)), avail_icu);
        avail_icu = avail_icu - new_icu;
        admit_to_icu(:, s, :) = new_icu;
    end
    %% Defer the excess burden to general wards.
    excess_icu = inc_icu - admit_to_icu;
    inc_ward = inc_ward + excess_icu;

    avail_ward = repmat(cap_ward, [data.lhs_data.samples 366]);
    admit_to_ward = zeros(size(inc_ward));
    for s = priority
        new_ward = min(squeeze(inc_ward(:, s, :)), avail_ward);
        avail_ward = avail_ward - new_ward;
        admit_to_ward(:, s, :) = new_ward;
    end
    %% Defer the excess burden to emergency departments.
    %% TODO: we are not converting incidence back to prevalence!
    excess_ward = inc_ward - admit_to_ward;
    inc_ed = inc_ed + excess_ward;

    avail_ed = repmat(cap_ed, [data.lhs_data.samples 366]);
    admit_to_ed = zeros(size(inc_ed));
    for s = priority
        new_ed = min(squeeze(inc_ed(:, s, :)), avail_ed);
        avail_ed = avail_ed - new_ed;
        admit_to_ed(:, s, :) = new_ed;
    end
    %% Defer the excess burden to GP clinics.
    excess_ed = inc_ed - admit_to_ed;
    inc_gp = inc_gp + excess_ed;

    avail_gp = repmat(cap_gp, [data.lhs_data.samples 366]);
    admit_to_gp = zeros(size(inc_gp));
    for s = priority
        new_gp = min(squeeze(inc_gp(:, s, :)), avail_gp);
        avail_gp = avail_gp - new_gp;
        admit_to_gp(:, s, :) = new_gp;
    end
    %% Count the excess burden on GP clinics.
    excess_gp = inc_gp - admit_to_gp;

    %% Now determine net burden for each health-care setting and other
    %% statistics (e.g., peak excess burdens, duration of excess).
    daily_load_icu = squeeze(sum(admit_to_icu, 2));
    daily_load_ward = squeeze(sum(admit_to_ward, 2));
    daily_load_ed = squeeze(sum(admit_to_ed, 2));
    daily_load_gp = squeeze(sum(admit_to_gp, 2));
    daily_excess_icu = squeeze(sum(excess_icu, 2));
    daily_excess_ward = squeeze(sum(excess_ward, 2));
    daily_excess_ed = squeeze(sum(excess_ed, 2));
    daily_excess_gp = squeeze(sum(excess_gp, 2));
    durn_excess_icu = sum(daily_excess_icu >= 1, 2);
    durn_excess_ward = sum(daily_excess_ward >= 1, 2);
    durn_excess_ed = sum(daily_excess_ed >= 1, 2);
    durn_excess_gp = sum(daily_excess_gp >= 1, 2);
    peak_excess_icu = max(daily_excess_icu, [], 2);
    peak_excess_ward = max(daily_excess_ward, [], 2);
    peak_excess_ed = max(daily_excess_ed, [], 2);
    peak_excess_gp = max(daily_excess_gp, [], 2);

    peak_excess = [peak_excess_icu, peak_excess_ward, ...
                   peak_excess_ed, peak_excess_gp];
    durn_excess = [durn_excess_icu, durn_excess_ward, ...
                   durn_excess_ed, durn_excess_gp];
    peak_median = median(peak_excess);
    peak_mean = mean(peak_excess);
    peak_qtls = quantile(peak_excess, [0.05 0.95]);
    durn_median = median(durn_excess);
    durn_mean = mean(durn_excess);
    durn_qtls = quantile(durn_excess, [0.05 0.95]);
    admissions = [sum(daily_load_icu, 2), sum(daily_load_ward, 2), ...
                  sum(daily_load_ed, 2), sum(daily_load_gp, 2)];
    admit_median = median(admissions);
    admit_qtls = quantile(admissions, [0.05, 0.95]);
    admit_mean = mean(admissions);

    peak_summary = dataset([[peak_median; peak_qtls; peak_mean], ...
                            strcat('ExPeak_', locn_names)], ...
                           'obsnames', obs_names);
    durn_summary = dataset([[durn_median; durn_qtls; durn_mean], ...
                            strcat('ExDurn_', locn_names)], ...
                           'obsnames', obs_names);
    admit_summary = dataset([[admit_median; admit_qtls; admit_mean], ...
                             strcat('Burden_', locn_names)], ...
                            'obsnames', obs_names);

    %% result = struct('Scenario', scenario, ...
    %%                 'Strategy', strategy, ...
    %%                 'CAP', CAP_summary, ...
    %%                 'deaths', death_summary, ...
    %%                 'excess_peak', peak_summary, ...
    %%                 'excess_durn', durn_summary, ...
    %%                 'AV_usage', av_summary, ...
    %%                 'AV_weeks', av_wk_summary, ...
    %%                 'CI_2weeks', ci_2wk_summary, ...
    %%                 'CI_4weeks', ci_4wk_summary ...
    %%                );

    names = [get(CAP_summary, 'VarNames'), ...
             get(death_summary, 'VarNames'), ...
             get(peak_summary, 'VarNames'), ...
             get(durn_summary, 'VarNames'), ...
             get(admit_summary, 'VarNames'), ...
             get(av_summary, 'VarNames'), ...
             get(av_wk_summary, 'VarNames'), ...
             get(ci_2wk_summary, 'VarNames'), ...
             get(ci_4wk_summary, 'VarNames'), ...
            ];

    values = [double(CAP_summary), double(death_summary), ...
              double(peak_summary), double(durn_summary), ...
              double(admit_summary), ...
              double(av_summary), double(av_wk_summary), ...
              double(ci_2wk_summary), double(ci_4wk_summary)];

    %% Return a structure that identifies the flu scenario and AV strategy,
    %% the names of the variables and statistics, and the actual values.
    result = struct('Scenario', scenario, ...
                    'Strategy', strategy, ...
                    'name', {names}, ...
                    'measure', {get(CAP_summary, 'ObsNames')}, ...
                    'value', {values});

    %%
    %% res = cell(7, 6);
    %% for S = 1:7
    %%     for A = 0:5
    %%         res{S, A + 1} = get_daily_inc(S, A);
    %%     end
    %% end
    %% save('summaries.mat', 'res', '-v7.3');
    %%
    %%
    %% %% Add two rows to hold the Scenario and Strategy.
    %% mat_size = size(res{1, 1}.value) + [2 0];
    %% get_tbl = @(sc, st) [res{sc, st + 1}.value; ...
    %%                      repmat(sc, 1, mat_size(2)); ...
    %%                      repmat(st, 1, mat_size(2));]';
    %% tbls = [get_tbl(1, 0); get_tbl(1, 1); get_tbl(1, 2); ...
    %%         get_tbl(1, 3); get_tbl(1, 4); get_tbl(1, 5); ...
    %%         get_tbl(2, 0); get_tbl(2, 1); get_tbl(2, 2); ...
    %%         get_tbl(2, 3); get_tbl(2, 4); get_tbl(2, 5); ...
    %%         get_tbl(3, 0); get_tbl(3, 1); get_tbl(3, 2); ...
    %%         get_tbl(3, 3); get_tbl(3, 4); get_tbl(3, 5); ...
    %%         get_tbl(4, 0); get_tbl(4, 1); get_tbl(4, 2); ...
    %%         get_tbl(4, 3); get_tbl(4, 4); get_tbl(4, 5); ...
    %%         get_tbl(5, 0); get_tbl(5, 1); get_tbl(5, 2); ...
    %%         get_tbl(5, 3); get_tbl(5, 4); get_tbl(5, 5); ...
    %%         get_tbl(6, 0); get_tbl(6, 1); get_tbl(6, 2); ...
    %%         get_tbl(6, 3); get_tbl(6, 4); get_tbl(6, 5); ...
    %%         get_tbl(7, 0); get_tbl(7, 1); get_tbl(7, 2); ...
    %%         get_tbl(7, 3); get_tbl(7, 4); get_tbl(7, 5);];
    %% name_lst = res{1,1}.name';
    %% name_lst(1:5) = strcat('CAP_', name_lst(1:5));
    %% name_lst(6:11) = strcat('Death_', name_lst(6:11));
    %% name_lst(12:15) = strcat('Peak_', name_lst(12:15));
    %% name_lst(16:19) = strcat('Durn_', name_lst(16:19));
    %% name_lst(20:25) = strcat('AV_', name_lst(20:25));
    %% name_lst(30:34) = strcat('CI2wk_', name_lst(30:34));
    %% name_lst(35:39) = strcat('CI4wk_', name_lst(35:39));
    %% name_col = repmat(name_lst, 42, 1);
    %% ctbl = [name_col, num2cell(tbls)];
    %% titles = {'stat', 'median', '5pcnt', '95pcnt', 'mean', 'scenario', ...
    %%           'strategy'};
    %% tbl = [titles; ctbl];
    %% fileID = fopen('summary_table.ssv','w');
    %% formatSpec = '%s %f %f %f %f %d %d\n';
    %% [nrows,ncols] = size(tbl);
    %% fprintf(fileID, '%s %s %s %s %s %s %s\n', tbl{1, :});
    %% for row = 2:nrows
    %%     fprintf(fileID, formatSpec, tbl{row, :});
    %% end
    %% fclose(fileID);
end
