%% function [mat_file] = master(flu_scenario, av_strategy, lhs_samples)
function [model, param_dists, samples, param1, curr_dir] = master(flu_scenario, av_strategy, lhs_samples)
    %MASTER  The master file that defines parameters and runs LHS experiments.
    %
    % [mat_file] = master(scenario, av_strategy, lhs_samples)
    %
    % Inputs:
    %   flu_scenario - The influenza scenario (1..7, see below).
    %   av_strategy  - The antiviral distribution strategy (1..5, see below).
    %   lhs_samples  - The (optional) number of LHS samples to simulate.
    %                  Default: 10,000 samples (only override when necessary).
    %
    % Output:
    %   mat_file - The name of the simulation output file.
    %
    % Flu Scenarios:
    %        Transmission    Severity
    %   -----------------------------
    %   1    Low             Low
    %   2    High            Low
    %   3    Low             Medium
    %   4    Medium          Medium
    %   5    High            Medium
    %   6    Low             High
    %   7    High            High
    %
    % AV Strategies:
    %        Treatment                         Prophylaxis
    %   ---------------------------------------------------------
    %   0    All cases                         All cases
    %   1    All cases                         High-risk, Elderly
    %   2    All cases                         None
    %   3    Children, High-risk, Elderly,     None
    %        HCWs, all hospital/ICU patients
    %   4    Hospital and ICU patients only    None
    %   5    None (*all* phases)               None (*all* phases)
    %

    curr_dir = pwd();
    exp_dir = fileparts(mfilename('fullpath'));
    model_dir = fileparts(exp_dir);

    %% Change directory to the one containing the model code.
    cd(model_dir);

    %% Load the model definition.
    model = discrete_model();

    %% Define the parameter distributions (overriding the defaults).
    param_dists = model.default_dists('PCR');
    %% Switch to syndromic diagnosis once diagnostic capacity is exceeded.
    param_dists.switchToSyndromic = dist_const(1);

    %% Define the transmission parameter (R0).
    if any(flu_scenario == [1 3 6])
        param_dists.R0 = dist_flat(1.05, 1.20);
    elseif any(flu_scenario == [4])
        param_dists.R0 = dist_flat(1.20, 1.40);
    elseif any(flu_scenario == [2 5 7])
        param_dists.R0 = dist_flat(1.40, 1.70);
    else
        throw(MException('Master Experiment', 'Invalid flu scenario'))
    end

    %% Define the severity parameter (eta).
    if any(flu_scenario == [1 2])
        param_dists.eta = dist_logflat(-4, -3);
    elseif any(flu_scenario == [3 4 5])
        param_dists.eta = dist_logflat(-3, -2);
    elseif any(flu_scenario == [6 7])
        param_dists.eta = dist_logflat(-2, -1);
    else
        throw(MException('Master Experiment', 'Invalid flu scenario'))
    end

    %% TODO: clinical care pathways.
    %%
    %%   Mild cases   - 80% to GP, 20% to ED
    %%   Severe cases - 87.5% to wards, 12.5% to ICUs (general population)
    %%                - 75% to wards, 25% to ICUs (at-risk population)
    %%   Deaths       - 5% of hospitalised cases (general population)
    %%                - 50% increase in risk of death for at-risk ICU cases
    %%
    %% ICU admission and death risks are concordant with FluCAN data.

    %% The relative propensity of "breakthrough" cases to display symptoms
    %% and subsequently report to medical services.
    param_dists.proph_relpathogenicity = dist_const(0.5);

    %% The efficacy of antiviral interventions.
    param_dists.e_t = dist_beta(0.7, 0.1, 0.5, 0.9);
    param_dists.e_s = dist_beta(0.7, 0.1, 0.4, 1.0);

    %% Binary indicator for treating severe cases, always true.
    param_dists.Rxsev = dist_const(1.0);

    %% Compliance at outpatient facilities (scales psi and epsilon).
    param_dists.mu = dist_flat(0.5, 1.0);

    %% The fraction of outpatient presentations to GP clinics.
    param_dists.frac_gp = dist_const(0.8);

    %% Determine which strata receive treatment when *not* hospitalised.
    %% Hospital and ICU patients are *always* treated (see Rxsev, above).
    if any(av_strategy == [0 1 2])
        %% Treat all cases.
        param_dists.treat_strata = dist_const(1);
    elseif av_strategy == 3
        %% Treat children, high-risk, elderly, and HCW cases.
        param_dists.treat_strata = dist_const([1; 1; 1; 1; 0;]);
        %% param_dists.treat_strata = dist_many({dist_const(1); dist_const(1);
        %%                                       dist_const(1); dist_const(0);
        %%                                       dist_const(0)});
    elseif av_strategy == 4
        %% Treat hospitalised cases only.
        param_dists.treat_strata = dist_const(0);
    elseif av_strategy == 5
        %% No antiviral stockpile.
        param_dists.O = dist_const(0);
    else
        throw(MException('Master Experiment', 'Invalid AV strategy'))
    end
    %% Determine which strata receive prophylaxis.
    if av_strategy == 0
        param_dists.proph_strata = dist_const(1);
    elseif av_strategy == 1
        param_dists.proph_strata = dist_const([0; 1; 1; 0; 0;]);
        %% param_dists.proph_strata = dist_wrapper(@(v) [0; 1; 1; 0; 0;], struct());
        %% param_dists.proph_strata = dist_many({dist_const(0); dist_const(1);
        %%                                       dist_const(1); dist_const(0);
        %%                                       dist_const(0)});
    else
        param_dists.proph_strata = dist_const(0);
    end

    %% Based on contact studies conducted by Edmunds and colleagues.
    param_dists.sigma = dist_flat(0.1, 0.5);

    %% The number of LHS samples.
    if exist('lhs_samples', 'var')
        samples = lhs_samples;
    else
        samples = 1e4;
    end

    param1 = struct(...
        'name', 'dummy', ...
        'description', 'dummy', ...
        'values', 1);

    %% Note: each scenario file must call LHS() directly, so that the output
    %%       file name will match that of the scenario file.
    %% [mat_file] = lhs(model, param_dists, samples, param1);

    %% Restore the previous working directory.
    %% cd(curr_dir);
end
