function [mat_file, lhs_data] = test_no_resistance(strategy)
    %% High transmission, high severity.
    scenario = 7;

    if (exist('strategy', 'var'))
        av_strategy = strategy;
    else
        %% Treat all mild cases, provide no prophylaxis.
        %% av_strategy = 2;
        %% Treat no mild cases, provide no prophylaxis.
        %% av_strategy = 4;
        %% Treat all mild cases, provide prophylaxis to high-risk and elderly.
        av_strategy = 1;
    end


    %% Run the simulation and load the results.
    [model, dists, samples, param1, prev_pwd] = master(scenario, av_strategy);
    %% We only want to run one or two model realisations.
    samples = 2;
    [mat_file] = lhs(model, dists, samples, param1);
    data = load(mat_file);
    lhs_data = data.lhs_data;

    %% Restore the previous working directory.
    cd(prev_pwd);

    [pathstr, test_name, ext] = fileparts(mfilename());

    if ~ all(lhs_data.parameter_samples.frac_gp == 0.8)
        error(test_name, 'frac_gp is not 80%');
    end

    for j = 1:samples
    %% Extract the weekly incidence time-series for each strata.
    weekly_incid = reshape(lhs_data.stats(1, j, 77:341), [5, 53]);
    maxI = max(weekly_incid, [], 2);
    popns = lhs_data.unsampled_params.N .* lhs_data.unsampled_params.pi;

    fprintf('Phase 1 started on day %.1f\n', lhs_data.stats(1, j, 61));
    fprintf('Phase 2 started on day %.1f\n', lhs_data.stats(1, j, 61) + 28);
    fprintf('Stockpile usage %f\n', lhs_data.stats(1, j, 62));
    fprintf('Stockpile expired on day %f\n', lhs_data.stats(1, j, 63));
    fprintf('AV strategy:            %1d\n', av_strategy);
    fprintf('AV doses:               %9.1f\n', lhs_data.stats(1, j, 66));
    fprintf('           Treatment    %9.1f\n', lhs_data.stats(1, j, 64));
    fprintf('           @ 2 weeks    %9.1f\n', lhs_data.stats(1, j, 70));
    fprintf('           @ 4 weeks    %9.1f\n', lhs_data.stats(1, j, 71));
    fprintf('            (Severe)    %9.1f\n', lhs_data.stats(1, j, 67));
    fprintf('              (Mild)    %9.1f\n', lhs_data.stats(1, j, 68));
    fprintf('      (Mild non-flu)    %9.1f\n', lhs_data.stats(1, j, 69));
    fprintf('         Prophylaxis    %9.1f\n', lhs_data.stats(1, j, 65));
    fprintf('           @ 2 weeks    %9.1f\n', lhs_data.stats(1, j, 72));
    fprintf('           @ 4 weeks    %9.1f\n', lhs_data.stats(1, j, 73));
    fprintf(' AR  ')
    fprintf('%6.3f', lhs_data.stats(1, j, 1:5));
    fprintf('\nCAR  ')
    fprintf('%6.3f', lhs_data.stats(1, j, 6:10));
    fprintf('\nMax Weekly Incidence    ');
    fprintf('%9.1f', maxI);
    fprintf('\n(Fraction of strata)    ');
    fprintf('%9.5f', maxI ./ popns);
    fprintf('\n\n');

    %% The data.lhs_data structure contains:
    %%    'param_dists'
    %%    'samples'
    %%    'stat_names'
    %%    'stats'
    %%    'parameter_samples'
    %%    'control_params'
    %%    'exp_name'
    %%    'mat_file'
    %%    'unsampled_params' --> fixed parameters that were not sampled
    %%    'stat_layout' --> describes how to interpret 'stats'
end

%% Stockpile usage 781512.731440
%% AV strategy:            0
%% AV doses:                781512.7
%%            Treatment     457596.1
%%            @ 2 weeks       6156.1
%%            @ 4 weeks      30453.4
%%             (Severe)     286004.8
%%               (Mild)      72831.5
%%       (Mild non-flu)      98759.9
%%          Prophylaxis     323916.6
%%            @ 2 weeks      13021.3
%%            @ 4 weeks      54118.2
%%  AR   0.655 0.655 0.655 0.655 0.655
%% CAR   0.145 0.145 0.179 0.145 0.145

%% Stockpile usage 759103.665158
%% AV strategy:            1
%% AV doses:                759103.7
%%            Treatment     457559.9
%%            @ 2 weeks       6156.1
%%            @ 4 weeks      30453.4
%%             (Severe)     285929.5
%%               (Mild)      72831.0
%%       (Mild non-flu)      98799.4
%%          Prophylaxis     301543.8
%%            @ 2 weeks      13021.3
%%            @ 4 weeks      54118.2
%%  AR   0.655 0.654 0.654 0.655 0.655
%% CAR   0.145 0.145 0.179 0.145 0.145

%% Stockpile usage 511954.357019
%% AV strategy:            2
%% AV doses:                511954.4
%%            Treatment     457836.1
%%            @ 2 weeks       6156.1
%%            @ 4 weeks      30453.4
%%             (Severe)     286162.2
%%               (Mild)      72762.7
%%       (Mild non-flu)      98911.3
%%          Prophylaxis      54118.2
%%            @ 2 weeks      13021.3
%%            @ 4 weeks      54118.2
%%  AR   0.655 0.655 0.655 0.655 0.655
%% CAR   0.145 0.145 0.179 0.145 0.145

%% Stockpile usage 479760.127531
%% AV strategy:            3
%% AV doses:                479760.1
%%            Treatment     425641.9
%%            @ 2 weeks       6156.1
%%            @ 4 weeks      30453.4
%%             (Severe)     286263.7
%%               (Mild)      64790.6
%%       (Mild non-flu)      74587.6
%%          Prophylaxis      54118.2
%%            @ 2 weeks      13021.3
%%            @ 4 weeks      54118.2
%%  AR   0.655 0.655 0.655 0.655 0.655
%% CAR   0.145 0.145 0.179 0.145 0.145

%% Stockpile usage 370866.567476
%% AV strategy:            4
%% AV doses:                370866.6
%%            Treatment     316748.3
%%            @ 2 weeks       6156.1
%%            @ 4 weeks      30453.4
%%             (Severe)     287048.1
%%               (Mild)       2887.6
%%       (Mild non-flu)      26812.6
%%          Prophylaxis      54118.2
%%            @ 2 weeks      13021.3
%%            @ 4 weeks      54118.2
%%  AR   0.657 0.657 0.657 0.657 0.657
%% CAR   0.145 0.145 0.180 0.145 0.145
