%% Model Parameters
%
% This module contains the parameter values for the model. The parameters
% are stored in a single structure, params, and can be accessed by name.
%
%   params = model_params();
%   params.psi_S = 0.0;
%   beta = params.beta;
%
function [params] = model_params()
    % Initialise the base parameter values
    %
    % The derived parameters are:
    %   beta (from R0)
    %   epsilon_i
    %   psi_i
    %
    %
    % dummy parameter for LHS dummy runs.
    %
    dummy=0;
    %

    N = 20e6; % Population of 20 million

    % The population is stratified based on June 2014 ABS statistics:
    %
    %   * 6.5% aged 0-4 (1.3 million);
    %   * 13.7% aged 66+ (2.74 million);
    %   * 10% of the rest (aged 5-65) are high-risk (1.6 million);
    %   * 325,000 healthcare workers (aged 5-65); and
    %   * The remainder are aged 5-65 (general population, 14.035 million).
    %
    pi = [0.065; 0.137; 0.08; 3.25e5 / N; 0];
    pi(5) = 1 - sum(pi);
    n = length(pi); % Number of groups.

    % Kappa is the per-person contact rate matrix (i.e., people per year),
    % assuming that the number of contacts is the same for each strata.
    num_contacts = 20;
    kappa = num_contacts * repmat(pi', n, 1) * 365;

    % Default to providing no prophylaxis and treating all cases.
    proph_strata = 0 * pi;
    treat_strata = proph_strata + 1;

    % Transmission Probability Matrix (tpm) (matrix): given contact between
    % S and I, probability of
    % transmission (TAKE CARE, WHICH WAY AROUND IS THIS?)
    tpm = ones(n);

    O = 0.4 * N; % Drug stockpile to cover 40% of the population

    omega = 365 / 1 * ones(n,1); % Latent phase of 1 day
    gamma = 365 / 1.5 * ones(n,1); % Infectious phase of 1.5 days
    chi = 1.0 * ones(n,1); % Relative infectivity of sub-clinical infections
    R0 = 1.38; % R0 to set the overall clinical + subclinical attack rate
    %R0scaling: determined in dep_dists() (contained in discrete_model.m)
    I0 = 100*pi; % Initial number of infections

    % Proportion of exposures resulting in infections
    proph_relpathogenicity = 0.5; % Relative proportion symptomatic for breakthrough infections

    % Proportion of infections that are severe (all of which present)
    eta = 0.1 * ones(n,1); % 1% are severe
    % The relative risk of severe cases for each strata.
    % The only non-unity risk is the elevated risk for the high-risk strata.
    eta_relscale = [1; 1; 5; 1; 1];
    % Proportion of *remaining* infections that present
    %alpha_m: Determined in dep_dists() (contained in discrete_model.m)
    %alpha_np_w, alpha_p_w, alpha_r: Determined in dep_dists()

    % Diagnosis regime parameters (fixed)
    % WARNING: Sn, Sp, Dc, e_dp, e_dt may be changed by calls to
    % diagnosis_dists() or diagnosis_regime()
    Sn = 1.0; % Diagnosis sensitivity; 1.0 for PCR tests, 0.56 for PoC tests
    Sp = 1.0; % 1.0 for PCR tests, 0.90-0.95 for PoC tests
    Dc = 1e4; % Capacity for outpatient testing
    e_dp = 0.7; % Delayed prophylaxis is only 70% effective
    e_dt = 0.7; % Delayed treatment is only 70% effective
    switchToSyndromic = 0; % Whether to switch to syndromic diagnosis once the
                           % diagnostic capacity is exceeded
    % Diagnosis regime parameters (variable)
    mu = zeros(n,1); % Compliance at outpatient facilities (scales psi and epsilon)
    sigma = zeros(n,1); % Fraction of contacts that receive prophylaxis
    Rxsev = zeros(n,1); % BINARY indicators - treat (1) or don't treat (0) severe cases
    frac_gp = 0.5 * ones(n,1); % Fraction of outpatient presentations at GPs

    % Resistant virus strains
    phi = 0.0; % Relative frequency of resistant strain infectives
    rho_t = 0.0; % Proportion of offspring of treated infectives who are resistant
    rho_p = 0.0; % Proportion of offspring of prophylaxis infectives who are resistant
    rho_n = 0.0; % Proportion of spontaneous resistance with antiviral selection pressure
    rho_n_factor = 1e-2; % Relative scale between rho_t and rho_n

    % Number of contacts
    delta = (365 / 3) * ones(n,1); % Removal from the contact class after 3 days

    % INTERVENTIONS
    % make sure AV and school interventions are OFF by default

    % Threshold for initiating the intervention
    start_intervention_rate = 10; % number of presentations required to initiate AV program

    % Effects of treatment and prophylaxis
    e_t = 0.95 * ones(n,1); % Relative infectiousness due to treatment
    e_s = 0.5 * ones(n,1); % Relative susceptibility due to prophylaxis
    e_i = 0.8 * ones(n,1); % Relative infectiousness of breakthrough cases

    % Health-care workers
    % Fixed pre-exposure prophylaxis drain on the stockpile
    HCWRate_Ph1 = 0;%300000*(365/10);
    HCWRate_Ph2 = 0;%300000*(365/10);
    HCWStop = 18/52; % HCW prophylaxis stop week (see VaccineStart below).

    % School based measures
    school_start = 0*52/365; % Start week for school based measures
    school_duration = 0*52/365; % Duration of school based measures
    school_efficacy = 0; % 0-1 (1=maximum efficacy, reduces M_{11})

    % Vaccination
    VaccineStart = 118/52; % Never begin delivering vaccine doses.
    VaccineDeliveryRate = 750000 * 52 .* pi; % 750,000 doses per week
    VaccineFailureRate = 0.3 * ones(n,1); % 30% of vaccinations have no effect
    VaccineSusceptibility = 0.3 * ones(n,1); % vaccine reduces susceptibility by 70%

    TimeNotCaseLoad = 1; % 1: a fixed Contain duration.
                         % 0: Contain ends at a fixed incidence
    ContainDuration = 28/365; % Fixed duration Contain
    ContainCaseLoad = 1000; % Peak Contain incidence (cases per day)

    % Antiviral delivery constraints are based on an "aspirational target" of
    % providing prophylaxis for 1,000 cases per day. Assuming that ~10 contacts
    % per case receive prophylaxis, the maximal delivery is 10^4 doses per day.
    % The constraint on treatment is much less important, since treatment has a
    % minimal effect, but the constraint is probably about the same magnitude
    % and thus the same value (10^4) is used.
    MaxAVtoTreat_Ph1 = 1e4; % Maximum AV packets distributed per day during Contain
    MaxAVtoTreat_Ph2 = 1e4; % ...and during Sustain
    MaxAVtoContacts_Ph1 = 1e4; % Maximum AV packets distributed per day during Contain
    MaxAVtoContacts_Ph2 = 1e4; % ...and during Sustain

    %% Consultation capacity constraints.
    %%   GP capacity - 171,000 influenza patients per day.
    cap_gp = 171000;
    %%   ED capacity -   8,900 consults per day.
    cap_ed = 8900;
    %%   Ward beds   -  27,600 beds for influenza patients (50% of all beds).
    cap_ward = 27600;
    %%   ICU beds    -   1,000 beds for influenza patients (50% of all beds).
    cap_icu = 1000;

    % Simulation time parameters
    time_start = 0;
    time_end = 1; % measured in years
    time_entries = 365*16; % resolution (16 steps per day, fewer CAN cuase issues)

    % Solve the model using ODE45 (continuously) or discretely (Euler,
    % suitable for stochastic models
    solve_discretely = 1;

    % Export the parameters in a single structure
    params = struct('N', N, 'pi', pi, ...
        'kappa', kappa, 'tpm', tpm, ...
        'O', O, 'omega', omega, 'gamma', gamma, ...
        'chi', chi, 'R0', R0, 'I0', I0, ...
        'proph_relpathogenicity', proph_relpathogenicity, ...
        'phi', phi, ...
        'rho_t', rho_t, 'rho_p', rho_p, 'rho_n', rho_n, 'delta', delta, ...
        'rho_n_factor', rho_n_factor, ...
        'start_intervention_rate', start_intervention_rate, ...
        'eta', eta, ...
        'eta_relscale', eta_relscale, ...
        'e_t', e_t, 'e_s', e_s, 'e_i', e_i, 'HCWStop', HCWStop, ...
        'HCWRate_Ph1', HCWRate_Ph1, 'HCWRate_Ph2', HCWRate_Ph2, ...
        'Sn', Sn, 'Sp', Sp, 'Dc', Dc, 'sigma', sigma, 'mu', mu, 'Rxsev', Rxsev, ...
        'switchToSyndromic', switchToSyndromic, ...
        'proph_strata', proph_strata, 'treat_strata', treat_strata, ...
        'cap_gp', cap_gp, 'cap_ed', cap_ed, ...
        'cap_ward', cap_ward, 'cap_icu', cap_icu, ...
        'e_dp', e_dp, 'e_dt', e_dt, ...
        'frac_gp', frac_gp, ...
        'SchoolStart', school_start, 'SchoolDuration', school_duration, ...
        'SchoolEfficacy', school_efficacy, ...
        'VaccineStart', VaccineStart, ...
        'VaccineDeliveryRate', VaccineDeliveryRate, ...
        'VaccineFailureRate', VaccineFailureRate, ...
        'VaccineSusceptibility', VaccineSusceptibility, ...
        'TimeNotCaseLoad', TimeNotCaseLoad, ...
        'ContainDuration', ContainDuration, ...
        'ContainCaseLoad', ContainCaseLoad, ...
        'MaxAVtoTreat_Ph1', MaxAVtoTreat_Ph1, ...
        'MaxAVtoTreat_Ph2', MaxAVtoTreat_Ph2, ...
        'MaxAVtoContacts_Ph1', MaxAVtoContacts_Ph1, ...
        'MaxAVtoContacts_Ph2', MaxAVtoContacts_Ph2, ...
        'time_start', time_start, 'time_end', time_end, ...
        'time_entries', time_entries, ...
        'solve_discretely', solve_discretely, 'dummy',dummy);
end
