% healthcare.m
%
% Analyses the model output and returns various statistics that relate to the
% load on the healthcare system and the delivery of vaccinations.
%
function [output] = model_statistics(time, params, state, events)
    if nargin == 2
        n = length(params.pi);
        output = struct('AttackRate', zeros(n,1), ...
            'ClinicalAttackRate', zeros(n,1), ...
            'CAR05', zeros(n,1), ...
            'CAR50', zeros(n,1), ...
            'CAR95', zeros(n,1), ...
            'EpidemicDuration', zeros(n,1), ...
            'TotalPresentations', zeros(n,1), ...
            'TotalNonPresentations', zeros(n,1), ...
            'TotalInfections', zeros(n,1), ...
            'TimeOfPeak', zeros(n,1), ...
            'MaximumDailyPandemicPresentations', zeros(n,1), ...
            'MaximumDailyILIPresentations', zeros(n,1), ...
            'TimeStartIntervention', 0, ...
            'StockpileUsage', 0, ...
            'StockpileExpiry',0, ...
            'AntiviralTreatmentDoses', 0, ...
            'AntiviralProphylaxisDoses',0, ...
            'AntiviralDoses', 0, ...
            'AntiviralSevereTreatmentDoses', 0, ...
            'AntiviralCommunityTreatmentDoses', 0, ...
            'AntiviralCommunityIneffectiveDoses', 0, ...
            'AntiviralTreatment2Weeks', 0, ...
            'AntiviralTreatment4Weeks', 0, ...
            'AntiviralProphylaxis2Weeks', 0, ...
            'AntiviralProphylaxis4Weeks', 0, ...
            'DiagnosticCapacityExceeded', 0, ...
            'VaccinesDelivered', 0, ...
            'VaccinesDeliveredtoSusceptibles', 0, ...
            %{
            %'NumberofResistantCases', zeros(n,1), ...
            %'ResistantCasesatExpiry', zeros(n,1), ...
            %'ResistantProportionatExpiry', zeros(n,1), ...
            %'StartofResistantEpidemic', zeros(n,1), ...
            %'DurationofResistantEpidemic', zeros(n,1), ...
            %}
            'IncidenceWeeklyPerWeek', zeros(n,floor(52*params.time_end-params.time_start)+1), ...
            'CumInc2Weeks', zeros(n,1), ...
            'CumInc4Weeks', zeros(n,1), ...
            'DailyIncNP', zeros(n,floor(365*params.time_end-params.time_start)+1), ...
            'DailyIncP', zeros(n,floor(365*params.time_end-params.time_start)+1) ...
            );
        return;
    elseif nargin ~= 4
        fprintf(1, 'ERROR: invalid # of parameters to healthcare\n');
        return;
    end

    % Fix shape of params.alpha_m (dep distribution limitation)
    params.alpha_m = params.alpha_m';

    % Record the following stats:
    %  > final attack rate and clinical attack rate
    %  > time of 10% 50% and 90% AR (R/N)
    %  > total and peak (per day) presentations (pand and ILI) and infections
    %  > stockpile usage, time of stockpile expiry
    %  > # of doses for treatment and prophylaxis
    %  > time when diagnosis capacity was exceeded (events.time_diagCapExceeded)
    %  > vaccine delivered to population, and to susceptibles
    %  > time bought and time bought per packet delivered
    %    (in comparison to baseline) -- not possible here?

    % Extract relevant state variables
    n = length(params.pi);
    %S = state(:,1:n)'; % 1 2
    %S_v = state(:,n+1:2*n)'; % 3 4

    %C_p_w = state(:,2*n+1:3*n)'; % 5 6
    %C_np_w = state(:,3*n+1:4*n)'; % 7 8
    %C_p_r = state(:,4*n+1:5*n)'; % 9 10
    %C_np_r = state(:,5*n+1:6*n)'; % 11 12

    E_p_w = state(:,6*n+1:7*n)'; % 13 14
    E_np_w = state(:,7*n+1:8*n)'; % 15 16
    E_p_r = state(:,8*n+1:9*n)'; % 17 18
    E_np_r = state(:,9*n+1:10*n)'; % 19 20

    %I_p_w = state(:,10*n+1:11*n)'; % 21 22
    %I_npt_w = state(:,11*n+1:12*n)'; % 23 24
    %I_npnt_w = state(:,12*n+1:13*n)'; % 25 26
    %I_p_r = state(:,13*n+1:14*n)'; % 27 28
    %I_np_r = state(:,14*n+1:15*n)'; % 29 30

    %A_p_w = state(:,15*n+1:16*n)'; % 31 32
    %A_np_w = state(:,16*n+1:17*n)'; % 33 34
    %A_p_r = state(:,17*n+1:18*n)'; % 35 36
    %A_np_r = state(:,18*n+1:19*n)'; % 37 38

    RI_p_w = state(:,19*n+1:20*n)'; % 39 40
    RI_np_w = state(:,20*n+1:21*n)'; % 41 42
    RI_p_r = state(:,21*n+1:22*n)'; % 43 44
    RI_np_r = state(:,22*n+1:23*n)'; % 45 46

    RA_p_w = state(:,23*n+1:24*n)'; % 47 48
    RA_np_w = state(:,24*n+1:25*n)'; % 49 50
    RA_p_r = state(:,25*n+1:26*n)'; % 51 52
    RA_np_r = state(:,26*n+1:27*n)'; % 53 54

    CumInc = state(:,27*n+1:28*n)'; % 55 56
    CumIncP = state(:,28*n+1:29*n)'; % 55 56

    O = state(:,29*n+1)'; % 57
    O_pPh1 = state(:,29*n+2)'; % 58
    O_tPh1 = state(:,29*n+3)'; % 59
    O_pPh2 = state(:,29*n+4)'; % 60
    O_tPh2 = state(:,29*n+5)'; % 61
    O_tSev = state(:,29*n+6)'; % 62
    O_tMld = state(:,29*n+7)'; % 63
    O_tWst = state(:,29*n+8)'; % 63

    VaccineesFromS = state(:,29*n+9:30*n+8)'; % 64 65
    TotalVaccinees = state(:,30*n+9:31*n+8)'; % 66 67


    % Presentation rates derived from the model parameters $\eta$ and $\alpha_M$.
    alpha_np_w = params.eta + params.alpha_m .* (1 - params.eta);
    alpha_p_w = params.eta + params.proph_relpathogenicity * params.alpha_m .* (1 - params.eta);
    alpha_r = alpha_np_w;

    % Determine the number of presentations and infections
    Recovered_pres = RI_p_w + RI_np_w + RI_p_r + RI_np_r;
    Recovered_nonpres = RA_p_w + RA_np_w + RA_p_r + RA_np_r;
    Present_tot = Recovered_pres(:,end);
    NonPresent_tot = Recovered_nonpres(:,end);
    Infected_tot = Present_tot + NonPresent_tot;

    % Calculate weekly incidence, reported weekly
    weekly_incidence_per_week = events.timescale/52*[CumInc(:,1), diff(CumInc,1,2)];
    weekly_incidence_per_week = weekly_incidence_per_week(:,1:events.timescale*7/365:end);

    % Pull out cumulative at 2 and 4 weeks post initiation of intervention
    indx = events.time_startPh1 + (14 / 365) * events.timescale;
    CumInc2Weeks = CumInc(:,indx);
    indx = events.time_startPh1 + (28 / 365) * events.timescale;
    CumInc4Weeks = CumInc(:,indx);

    % Calculate daily incidence
    daily_incidence = events.timescale/365*[CumInc(:,1), diff(CumInc,1,2)];
    daily_incidence = daily_incidence(:, 1:events.timescale / 365:end);

    % Calculate daily incidence of cases who received prophylaxis
    daily_incidence_p = events.timescale/365*[CumIncP(:,1), diff(CumIncP,1,2)];
    daily_incidence_p = daily_incidence_p(:, 1:events.timescale / 365:end);

    % Calculate daily incidence of cases who did not receive prophylaxis
    daily_incidence_np = daily_incidence - daily_incidence_p;

    % Calculate attack rate and clinical attack rate
    AR = Infected_tot ./ (params.N*params.pi);
    CAR = Present_tot ./ (params.N*params.pi);

    % Times when the CAR was at 10%, 50% and 90% of the final value
    tCAR_05 = zeros(n,1);
    tCAR_50 = zeros(n,1);
    tCAR_95 = zeros(n,1);
    for i=1:n
        tCAR_05(i) = (365/events.timescale) ...
            * time(find(Recovered_pres(i,:) / (params.N*params.pi(i)) >= 0.05 * CAR(i), 1));
        tCAR_50(i) = (365/events.timescale) ...
            * time(find(Recovered_pres(i,:) / (params.N*params.pi(i)) >= 0.50 * CAR(i), 1));
        tCAR_95(i) = (365/events.timescale) ...
            * time(find(Recovered_pres(i,:) / (params.N*params.pi(i)) >= 0.95 * CAR(i), 1));
    end
    duration = tCAR_95 - tCAR_05;

    % Percentage of all ILI presentations that are infected
    % (presn_rate is converted from per year to per week)
    Pand_pcnt = @(presn_rate) min(0.65, max(0.03, sum(presn_rate,1) .* 3e-5 ./ 52));
    % Maximum daily presentation rates
    presn_rate_p = repmat(alpha_p_w .* params.omega,1,events.timescale+1) .* E_p_w;
    presn_rate_np = repmat(alpha_np_w .* params.omega,1,events.timescale+1) .* E_np_w;
    presn_rate_p_r = repmat(alpha_r .* params.omega,1,events.timescale+1) .* E_p_r;
    presn_rate_np_r = repmat(alpha_r .* params.omega,1,events.timescale+1) .* E_np_r;
    presn_rate = presn_rate_p + presn_rate_np + presn_rate_p_r + presn_rate_np_r;

    ILI_rate = presn_rate ./ repmat(Pand_pcnt(presn_rate),n,1);
    [max_daily_ILI_presns,indx] = max(ILI_rate,[],2);
    max_daily_ILI_presns = max_daily_ILI_presns / 365;
    [max_daily_pand_presns,~] = max(presn_rate,[],2);
    max_daily_pand_presns = max_daily_pand_presns / 365;
    time_of_peak = indx / (events.timescale/365);

    % Initiation of intervention
    time_startPh1 = events.time_startPh1 / (events.timescale/365);

    % Stockpile usage and diagnostic capacity
    stockpile_used = O(1) - min(O);
    stockpile_expired = 365 * events.time_stockpileExpired;
    diag_capacity_exceeded = 365 * events.time_diagCapExceeded;
    % Check whether these events actually occurred
    if stockpile_expired < 0
        stockpile_expired = -1; % NaN?
    end
    if diag_capacity_exceeded < 0
        diag_capacity_exceeded = -1; % NaN?
    end

    % Antiviral delivery
    treat_doses = max(O_tPh1 + O_tPh2);
    proph_doses = max(O_pPh1 + O_pPh2);
    total_doses = treat_doses + proph_doses;

    % Antivirals at 2 and 4 weeks
    temp_treat_doses = O_tPh1 + O_tPh2;
    temp_proph_doses = O_pPh1 + O_pPh2;

    indx = events.time_startPh1 + (14 / 365) * events.timescale + 1;
    treat_2weeks = temp_treat_doses(indx);
    proph_2weeks = temp_proph_doses(indx);
    indx = events.time_startPh1 + (28 / 365) * events.timescale + 1;
    treat_4weeks = temp_treat_doses(indx);
    proph_4weeks = temp_proph_doses(indx);

    % Drug resistant strain
    %resistant_cases = max(RI_p_r + RI_np_r + RA_p_r + RA_np_r);

    % Find the longest sequence of drug-resistant cases
    %res_epi_vec = (I_p_r + I_np_r + A_p_r + A_np_r) > 0;
    %[res_start, res_end] = longest_seqn(sum(res_epi_vec,2));
    %if isempty(res_start)
    %    res_epi_start = NaN;
    %    res_epi_duration = NaN;
    %else
    %    res_epi_start = time(res_start)';
    %    res_epi_duration = time(res_end)' - time(res_start)';
    %end

    % Vaccine usage
    Tot_vacc = sum(TotalVaccinees(:,end));
    Sus_vacc = sum(VaccineesFromS(:,end));

    % Collect all of the statistics
    output.AttackRate = AR;
    output.ClinicalAttackRate = CAR;
    output.CAR05 = tCAR_05;
    output.CAR50 = tCAR_50;
    output.CAR95 = tCAR_95;
    output.EpidemicDuration = duration;
    output.TotalPresentations = Present_tot;
    output.TotalNonPresentations = NonPresent_tot;
    output.TotalInfections = Infected_tot;
    output.TimeOfPeak = time_of_peak;
    output.MaximumDailyPandemicPresentations = max_daily_pand_presns;
    output.MaximumDailyILIPresentations = max_daily_ILI_presns;
    output.TimeStartIntervention = time_startPh1;
    output.StockpileUsage = stockpile_used;
    output.StockpileExpiry = stockpile_expired;
    output.AntiviralTreatmentDoses = treat_doses;
    output.AntiviralProphylaxisDoses = proph_doses;
    output.AntiviralDoses = total_doses;
    % The following two calculations aren't correct under all scenarios ...
    % output.AntiviralSevereTreatmentDoses = sum(params.eta .* AR .* params.pi * params.N);
    % output.AntiviralCommunityTreatmentDoses = treat_doses - sum(params.eta .* AR .* params.pi * params.N);
    output.AntiviralSevereTreatmentDoses = max(O_tSev);
    output.AntiviralCommunityTreatmentDoses = max(O_tMld);
    output.AntiviralCommunityIneffectiveDoses = max(O_tWst);
    output.AntiviralTreatment2Weeks = treat_2weeks;
    output.AntiviralTreatment4Weeks = treat_4weeks;
    output.AntiviralProphylaxis2Weeks = proph_2weeks;
    output.AntiviralProphylaxis4Weeks = proph_4weeks;
    output.DiagnosticCapacityExceeded = diag_capacity_exceeded;
    output.VaccinesDelivered = Tot_vacc;
    output.VaccinesDeliveredtoSusceptibles = Sus_vacc;
    %output.NumberofResistantCases = resistant_cases;
    %output.ResistantCasesatExpiry = events.resCasesAtExpiry';
    %output.ResistantProportionatExpiry = events.resPropnAtExpiry';
    %output.StartofResistantEpidemic = res_epi_start;
    %output.DurationofResistantEpidemic = res_epi_duration;
    output.IncidenceWeeklyPerWeek = weekly_incidence_per_week;
    output.CumInc2Weeks = CumInc2Weeks;
    output.CumInc4Weeks = CumInc4Weeks;
    output.DailyIncNP = daily_incidence_np;
    output.DailyIncP = daily_incidence_p;
end
